"""Unit tests for conversations API routes."""
import pytest
import json
from datetime import datetime
from unittest.mock import Mock, patch, mock_open
from pathlib import Path

from fastapi.testclient import TestClient
from fastapi import HTTPException

from searchat.api.app import app


@pytest.fixture
def client():
    """FastAPI test client."""
    return TestClient(app)


@pytest.fixture
def mock_duckdb_store():
    """Mock DuckDBStore used by conversations routes."""
    mock = Mock()

    now = datetime.now()
    mock._data = [
        {
            "conversation_id": "conv-1",
            "project_id": "project-a",
            "title": "Python Binary Search",
            "created_at": now,
            "updated_at": now,
            "message_count": 10,
            "file_path": "/home/user/.claude/conv-1.jsonl",
            "full_text": "This is a conversation about implementing binary search in Python...",
        },
        {
            "conversation_id": "conv-2",
            "project_id": "project-b",
            "title": "API Design",
            "created_at": now,
            "updated_at": now,
            "message_count": 5,
            "file_path": "C:\\Users\\Test\\.claude\\conv-2.jsonl",
            "full_text": "Discussion about REST API design patterns",
        },
        {
            "conversation_id": "conv-3",
            "project_id": "project-a",
            "title": "Empty Conversation",
            "created_at": now,
            "updated_at": now,
            "message_count": 0,
            "file_path": "/home/user/.claude/conv-3.jsonl",
            "full_text": "",
        },
    ]

    def _list_conversations(
        *,
        sort_by="length",
        project_id=None,
        date_from=None,
        date_to=None,
        limit=None,
        offset=0,
    ):
        rows = [r for r in mock._data if r["message_count"] > 0]
        if project_id:
            rows = [r for r in rows if r["project_id"] == project_id]
        if date_from:
            rows = [r for r in rows if r["updated_at"] >= date_from]
        if date_to:
            rows = [r for r in rows if r["updated_at"] < date_to]

        if sort_by == "length":
            rows.sort(key=lambda r: r["message_count"], reverse=True)
        elif sort_by == "date_newest":
            rows.sort(key=lambda r: r["updated_at"], reverse=True)
        elif sort_by == "date_oldest":
            rows.sort(key=lambda r: r["updated_at"], reverse=False)
        elif sort_by == "title":
            rows.sort(key=lambda r: r["title"])

        if offset:
            rows = rows[int(offset):]
        if limit is not None:
            rows = rows[: int(limit)]

        return rows

    def _count_conversations(*, project_id=None, date_from=None, date_to=None):
        rows = [r for r in mock._data if r["message_count"] > 0]
        if project_id:
            rows = [r for r in rows if r["project_id"] == project_id]
        if date_from:
            rows = [r for r in rows if r["updated_at"] >= date_from]
        if date_to:
            rows = [r for r in rows if r["updated_at"] < date_to]
        return len(rows)

    def _get_meta(conversation_id: str):
        for r in mock._data:
            if r["conversation_id"] == conversation_id:
                return {
                    "conversation_id": r["conversation_id"],
                    "project_id": r["project_id"],
                    "title": r["title"],
                    "created_at": r["created_at"],
                    "updated_at": r["updated_at"],
                    "message_count": r["message_count"],
                    "file_path": r["file_path"],
                }
        return None

    mock.list_conversations.side_effect = _list_conversations
    mock.count_conversations.side_effect = _count_conversations
    mock.get_conversation_meta.side_effect = _get_meta
    return mock


def test_extract_vibe_messages_filters_roles_and_content():
    from searchat.api.routers import conversations as conv

    data = {
        "messages": [
            {"role": "user", "content": "Hello", "timestamp": "2025-01-01T00:00:00"},
            {"role": "assistant", "text": "Hi"},
            {"role": "system", "content": "skip"},
            {"role": "user", "content": "   "},
            {"role": "assistant", "content": 123},
        ]
    }

    messages = conv._extract_vibe_messages(data)
    assert [m.role for m in messages] == ["user", "assistant"]
    assert [m.content for m in messages] == ["Hello", "Hi"]


def test_load_opencode_parts_text_reads_text_and_state(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    parts_dir = tmp_path / "storage" / "part" / "msg-1"
    parts_dir.mkdir(parents=True)
    (parts_dir / "000.json").write_text(json.dumps({"text": "alpha"}))
    (parts_dir / "001.json").write_text(json.dumps({"state": {"output": "beta"}}))

    assert conv._load_opencode_parts_text(tmp_path, "msg-1") == "alpha\n\nbeta"


def test_extract_opencode_message_text_prefers_content_and_summary(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    data_root = tmp_path
    assert (
        conv._extract_opencode_message_text(data_root, {"content": {"text": " content "}})
        == "content"
    )
    assert (
        conv._extract_opencode_message_text(data_root, {"content": [{"text": "a"}, {"content": "b"}]})
        == "a\n\nb"
    )

    parts_dir = data_root / "storage" / "part" / "msg-2"
    parts_dir.mkdir(parents=True)
    (parts_dir / "000.json").write_text(json.dumps({"text": "from parts"}))
    assert conv._extract_opencode_message_text(data_root, {"id": "msg-2"}) == "from parts"

    assert (
        conv._extract_opencode_message_text(data_root, {"summary": {"title": "Summary"}})
        == "Summary"
    )


def test_resolve_opencode_data_root(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    storage_path = tmp_path / "storage" / "session" / "proj" / "session.json"
    storage_path.parent.mkdir(parents=True)

    assert conv._resolve_opencode_data_root(str(storage_path)) == tmp_path

    deep_path = tmp_path / "a" / "b" / "c" / "d" / "file.json"
    deep_path.parent.mkdir(parents=True)
    assert conv._resolve_opencode_data_root(str(deep_path)) == deep_path.parents[3]


def test_load_opencode_and_vibe_project_paths(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    session_path = tmp_path / "storage" / "session" / "proj" / "session.json"
    session_path.parent.mkdir(parents=True)
    session_path.write_text(json.dumps({"projectID": "proj-1"}))

    project_path = tmp_path / "storage" / "project" / "proj-1.json"
    project_path.parent.mkdir(parents=True)
    project_path.write_text(json.dumps({"worktree": "/work/tree"}))

    assert conv._load_opencode_project_path(str(session_path)) == "/work/tree"

    vibe_path = tmp_path / "vibe.json"
    vibe_path.write_text(
        json.dumps({"metadata": {"environment": {"working_directory": "/vibe/project"}}})
    )
    assert conv._load_vibe_project_path(str(vibe_path)) == "/vibe/project"


def test_resolve_opencode_data_root_falls_back_to_parent(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    assert conv._resolve_opencode_data_root("session.json") == Path(".")


def test_load_opencode_project_path_invalid_inputs_return_none(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    bad_json = tmp_path / "bad.json"
    bad_json.write_text("not-json")
    assert conv._load_opencode_project_path(str(bad_json)) is None

    missing_project_id = tmp_path / "missing_project_id.json"
    missing_project_id.write_text("{}")
    assert conv._load_opencode_project_path(str(missing_project_id)) is None


def test_load_opencode_project_path_missing_or_invalid_project_file_returns_none(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    session_path = tmp_path / "storage" / "session" / "proj" / "session.json"
    session_path.parent.mkdir(parents=True)
    session_path.write_text(json.dumps({"projectID": "proj-1"}))

    assert conv._load_opencode_project_path(str(session_path)) is None

    project_path = tmp_path / "storage" / "project" / "proj-1.json"
    project_path.parent.mkdir(parents=True)
    project_path.write_text("not-json")
    assert conv._load_opencode_project_path(str(session_path)) is None

    project_path.write_text(json.dumps({"worktree": "   "}))
    assert conv._load_opencode_project_path(str(session_path)) is None


def test_load_vibe_project_path_missing_or_invalid_returns_none(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    bad_json = tmp_path / "bad.json"
    bad_json.write_text("not-json")
    assert conv._load_vibe_project_path(str(bad_json)) is None

    missing = tmp_path / "missing.json"
    missing.write_text(json.dumps({"metadata": {"environment": {}}}))
    assert conv._load_vibe_project_path(str(missing)) is None

    empty = tmp_path / "empty.json"
    empty.write_text(json.dumps({"metadata": {"environment": {"working_directory": "  "}}}))
    assert conv._load_vibe_project_path(str(empty)) is None


def test_messages_to_lines_includes_blank_lines() -> None:
    from searchat.api.routers import conversations as conv
    from searchat.api.models import ConversationMessage

    lines = conv._messages_to_lines([
        ConversationMessage(role="user", content="", timestamp=""),
        ConversationMessage(role="assistant", content="a\n\n", timestamp=""),
    ])
    assert "USER #1" in lines[0]
    assert "ASSISTANT #2" in lines
    assert "" in lines


def test_detect_language_various() -> None:
    from searchat.api.routers import conversations as conv

    assert conv._detect_language("SELECT 1") == "sql"
    assert conv._detect_language("def hi():\n  return 1\n") == "python"
    assert conv._detect_language("function x() { console.log('hi') }") == "javascript"
    assert conv._detect_language("type X = { a: string };\nconst x: X = { a: 'b' };") == "typescript"
    assert conv._detect_language("#!/bin/bash\necho hi\n") == "bash"
    assert conv._detect_language('{"a": 1}') == "json"
    assert conv._detect_language("{bad: 1}") == "plaintext"
    assert conv._detect_language("<div>hi</div>") == "html"
    assert conv._detect_language(".a { color: red; }") == "css"
    assert conv._detect_language("package main\nfunc main(){}\n") == "go"
    assert conv._detect_language("fn main() { println!(\"hi\"); }\n") == "rust"
    assert conv._detect_language("public static void main(String[] args) {}") == "java"
    assert conv._detect_language("hello") == "plaintext"


@pytest.mark.asyncio
async def test_load_opencode_messages_from_storage(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    session_id = "session-1"
    session_path = tmp_path / "storage" / "session" / "proj" / "session.json"
    session_path.parent.mkdir(parents=True)
    session_path.write_text(json.dumps({"messages": []}))

    messages_dir = tmp_path / "storage" / "message" / session_id
    messages_dir.mkdir(parents=True)
    (messages_dir / "000.json").write_text(
        json.dumps(
            {
                "role": "user",
                "content": "Hello",
                "time": {"created": 1_700_000_000_000},
            }
        )
    )

    with patch("searchat.api.routers.conversations.deps.get_config", return_value=Mock()):
        with patch("searchat.config.PathResolver.resolve_opencode_dirs", return_value=[]):
            messages = await conv._load_opencode_messages(str(session_path), session_id)

    assert len(messages) == 1
    assert messages[0].role == "user"
    assert messages[0].content == "Hello"
    assert messages[0].timestamp


@pytest.mark.asyncio
async def test_load_opencode_messages_from_session_file(tmp_path: Path):
    from searchat.api.routers import conversations as conv

    session_id = "session-2"
    session_path = tmp_path / "session.json"
    session_path.write_text(
        json.dumps(
            {
                "messages": [
                    {
                        "role": "assistant",
                        "content": {"text": "From content"},
                        "time": {"created": 1_700_000_000_500},
                    },
                    {"role": "user", "text": "Plain"},
                ]
            }
        )
    )

    with patch("searchat.api.routers.conversations.deps.get_config", return_value=Mock()):
        with patch("searchat.config.PathResolver.resolve_opencode_dirs", return_value=[]):
            messages = await conv._load_opencode_messages(str(session_path), session_id)

    assert [m.role for m in messages] == ["user", "assistant"]
    assert [m.content for m in messages] == ["Plain", "From content"]


@pytest.fixture
def mock_platform_manager():
    """Mock PlatformManager for terminal operations."""
    mock = Mock()
    mock.platform = "windows"
    mock.normalize_path = Mock(side_effect=lambda x: x)  # Return path unchanged
    mock.open_terminal_with_command = Mock()
    return mock


def test_conversation_diff_requires_search_engine(client):
    """Test diff endpoint returns 503 when search engine not ready."""
    with patch('searchat.api.routers.conversations.deps.get_search_engine', side_effect=RuntimeError("Search engine not ready")):
        response = client.get("/api/conversation/conv-1/diff")

        assert response.status_code == 503
        assert response.json()["detail"] == "Search engine not ready"


@pytest.mark.unit
class TestGetAllConversationsEndpoint:
    """Tests for GET /api/conversations/all endpoint."""

    def test_get_all_conversations_default_sort(self, client, mock_duckdb_store):
        """Test getting all conversations with default sort (by length)."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all")

            assert response.status_code == 200
            data = response.json()

            assert "results" in data
            assert "total" in data
            assert data["total"] == 2  # conv-3 filtered out (0 messages)

            # Should be sorted by message_count descending (conv-1: 10, conv-2: 5)
            assert data["results"][0]["conversation_id"] == "conv-1"
            assert data["results"][0]["message_count"] == 10
            assert data["results"][1]["conversation_id"] == "conv-2"
            assert data["results"][1]["message_count"] == 5

    def test_get_all_conversations_filters_zero_messages(self, client, mock_duckdb_store):
        """Test that conversations with 0 messages are filtered out."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all")

            assert response.status_code == 200
            data = response.json()

            # conv-3 has 0 messages and should be filtered
            conv_ids = [r["conversation_id"] for r in data["results"]]
            assert "conv-3" not in conv_ids
            assert len(data["results"]) == 2

    def test_get_all_conversations_sort_by_length(self, client, mock_duckdb_store):
        """Test sorting by message count (length)."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?sort_by=length")

            assert response.status_code == 200
            data = response.json()

            # Descending order by message count
            assert data["results"][0]["message_count"] >= data["results"][1]["message_count"]

    def test_get_all_conversations_sort_by_date_newest(self, client, mock_duckdb_store):
        """Test sorting by newest date."""
        now = datetime.now()
        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-old",
                "project_id": "project-a",
                "title": "Old",
                "created_at": now,
                "updated_at": datetime(2025, 1, 1),
                "message_count": 5,
                "file_path": "/test/old.jsonl",
                "full_text": "Old conversation",
            },
            {
                "conversation_id": "conv-new",
                "project_id": "project-a",
                "title": "New",
                "created_at": now,
                "updated_at": datetime(2025, 1, 31),
                "message_count": 5,
                "file_path": "/test/new.jsonl",
                "full_text": "New conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?sort_by=date_newest")

            assert response.status_code == 200
            data = response.json()

            # Newest first
            assert data["results"][0]["conversation_id"] == "conv-new"
            assert data["results"][1]["conversation_id"] == "conv-old"

    def test_get_all_conversations_sort_by_date_oldest(self, client, mock_duckdb_store):
        """Test sorting by oldest date."""
        now = datetime.now()
        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-old",
                "project_id": "project-a",
                "title": "Old",
                "created_at": now,
                "updated_at": datetime(2025, 1, 1),
                "message_count": 5,
                "file_path": "/test/old.jsonl",
                "full_text": "Old conversation",
            },
            {
                "conversation_id": "conv-new",
                "project_id": "project-a",
                "title": "New",
                "created_at": now,
                "updated_at": datetime(2025, 1, 31),
                "message_count": 5,
                "file_path": "/test/new.jsonl",
                "full_text": "New conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?sort_by=date_oldest")

            assert response.status_code == 200
            data = response.json()

            # Oldest first
            assert data["results"][0]["conversation_id"] == "conv-old"
            assert data["results"][1]["conversation_id"] == "conv-new"

    def test_get_all_conversations_sort_by_title(self, client, mock_duckdb_store):
        """Test sorting by title alphabetically."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?sort_by=title")

            assert response.status_code == 200
            data = response.json()

            # Alphabetical order
            titles = [r["title"] for r in data["results"]]
            assert titles == sorted(titles)

    def test_get_all_conversations_source_detection(self, client, mock_duckdb_store):
        """Test that source (WSL/WIN) is detected correctly."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all")

            assert response.status_code == 200
            data = response.json()

            # conv-1 has /home/ path (WSL)
            conv1 = next(r for r in data["results"] if r["conversation_id"] == "conv-1")
            assert conv1["source"] == "WSL"

            # conv-2 has C:\ path (Windows)
            conv2 = next(r for r in data["results"] if r["conversation_id"] == "conv-2")
            assert conv2["source"] == "WIN"

    def test_get_all_conversations_snippet_truncation(self, client, mock_duckdb_store):
        """Test that long text is truncated to snippet."""
        long_text = "A" * 300  # 300 characters
        now = datetime.now()

        mock_duckdb_store._data = [{
            "conversation_id": "conv-long",
            "project_id": "test",
            "title": "Long",
            "created_at": now,
            "updated_at": now,
            "message_count": 5,
            "file_path": "/test/long.jsonl",
            "full_text": long_text,
        }]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all")

            assert response.status_code == 200
            data = response.json()

            snippet = data["results"][0]["snippet"]
            assert len(snippet) == 203  # 200 chars + "..."
            assert snippet.endswith("...")

    def test_get_all_conversations_error_handling(self, client):
        """Test error handling when store fails."""
        mock_store = Mock()
        mock_store.list_conversations.side_effect = Exception("Database error")

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_store):
            response = client.get("/api/conversations/all")

            assert response.status_code == 500
            assert "Database error" in response.json()["detail"]

    def test_get_all_conversations_filter_by_project(self, client, mock_duckdb_store):
        """Test filtering conversations by project."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?project=project-a")

            assert response.status_code == 200
            data = response.json()

            # Only project-a conversations (conv-3 has 0 messages so filtered out)
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-1"
            assert data["results"][0]["project_id"] == "project-a"

    def test_get_all_conversations_filter_by_project_no_results(self, client, mock_duckdb_store):
        """Test filtering by project with no matching conversations."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?project=nonexistent-project")

            assert response.status_code == 200
            data = response.json()

            assert data["total"] == 0
            assert len(data["results"]) == 0

    def test_get_all_conversations_pagination_limit_offset(self, client, mock_duckdb_store):
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?sort_by=length&limit=1&offset=1")

            assert response.status_code == 200
            data = response.json()

            assert data["total"] == 2
            assert len(data["results"]) == 1
            assert data["results"][0]["conversation_id"] == "conv-2"

    def test_get_all_conversations_filter_by_date_today(self, client, mock_duckdb_store):
        """Test filtering conversations from today."""
        from datetime import timedelta

        now = datetime.now()
        yesterday = now - timedelta(days=1)

        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-today",
                "project_id": "project-a",
                "title": "Today",
                "created_at": now,
                "updated_at": now,
                "message_count": 5,
                "file_path": "/test/today.jsonl",
                "full_text": "Today conversation",
            },
            {
                "conversation_id": "conv-yesterday",
                "project_id": "project-a",
                "title": "Yesterday",
                "created_at": yesterday,
                "updated_at": yesterday,
                "message_count": 5,
                "file_path": "/test/yesterday.jsonl",
                "full_text": "Yesterday conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?date=today")

            assert response.status_code == 200
            data = response.json()

            # Should only have today's conversation
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-today"

    def test_get_all_conversations_filter_by_date_week(self, client, mock_duckdb_store):
        """Test filtering conversations from last 7 days."""
        from datetime import timedelta

        now = datetime.now()
        five_days_ago = now - timedelta(days=5)
        ten_days_ago = now - timedelta(days=10)

        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-recent",
                "project_id": "project-a",
                "title": "Recent",
                "created_at": five_days_ago,
                "updated_at": five_days_ago,
                "message_count": 5,
                "file_path": "/test/recent.jsonl",
                "full_text": "Recent conversation",
            },
            {
                "conversation_id": "conv-old",
                "project_id": "project-a",
                "title": "Old",
                "created_at": ten_days_ago,
                "updated_at": ten_days_ago,
                "message_count": 5,
                "file_path": "/test/old.jsonl",
                "full_text": "Old conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?date=week")

            assert response.status_code == 200
            data = response.json()

            # Should only have conversation from last 7 days
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-recent"

    def test_get_all_conversations_filter_by_date_month(self, client, mock_duckdb_store):
        """Test filtering conversations from last 30 days."""
        from datetime import timedelta

        now = datetime.now()
        twenty_days_ago = now - timedelta(days=20)
        forty_days_ago = now - timedelta(days=40)

        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-recent",
                "project_id": "project-a",
                "title": "Recent",
                "created_at": twenty_days_ago,
                "updated_at": twenty_days_ago,
                "message_count": 5,
                "file_path": "/test/recent.jsonl",
                "full_text": "Recent conversation",
            },
            {
                "conversation_id": "conv-old",
                "project_id": "project-a",
                "title": "Old",
                "created_at": forty_days_ago,
                "updated_at": forty_days_ago,
                "message_count": 5,
                "file_path": "/test/old.jsonl",
                "full_text": "Old conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?date=month")

            assert response.status_code == 200
            data = response.json()

            # Should only have conversation from last 30 days
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-recent"

    def test_get_all_conversations_filter_by_custom_date_range(self, client, mock_duckdb_store):
        """Test filtering conversations with custom date range."""
        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-jan-10",
                "project_id": "project-a",
                "title": "Jan 10",
                "created_at": datetime(2025, 1, 10),
                "updated_at": datetime(2025, 1, 10),
                "message_count": 5,
                "file_path": "/test/jan10.jsonl",
                "full_text": "Jan 10 conversation",
            },
            {
                "conversation_id": "conv-jan-15",
                "project_id": "project-a",
                "title": "Jan 15",
                "created_at": datetime(2025, 1, 15),
                "updated_at": datetime(2025, 1, 15),
                "message_count": 5,
                "file_path": "/test/jan15.jsonl",
                "full_text": "Jan 15 conversation",
            },
            {
                "conversation_id": "conv-jan-20",
                "project_id": "project-a",
                "title": "Jan 20",
                "created_at": datetime(2025, 1, 20),
                "updated_at": datetime(2025, 1, 20),
                "message_count": 5,
                "file_path": "/test/jan20.jsonl",
                "full_text": "Jan 20 conversation",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?date=custom&date_from=2025-01-12&date_to=2025-01-18")

            assert response.status_code == 200
            data = response.json()

            # Should only have Jan 15 (between Jan 12 and Jan 18 inclusive)
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-jan-15"

    def test_get_all_conversations_filter_combined(self, client, mock_duckdb_store):
        """Test filtering by both project and date."""
        from datetime import timedelta

        now = datetime.now()
        yesterday = now - timedelta(days=1)

        mock_duckdb_store._data = [
            {
                "conversation_id": "conv-a-today",
                "project_id": "project-a",
                "title": "A Today",
                "created_at": now,
                "updated_at": now,
                "message_count": 5,
                "file_path": "/test/a-today.jsonl",
                "full_text": "Project A today",
            },
            {
                "conversation_id": "conv-a-yesterday",
                "project_id": "project-a",
                "title": "A Yesterday",
                "created_at": yesterday,
                "updated_at": yesterday,
                "message_count": 5,
                "file_path": "/test/a-yesterday.jsonl",
                "full_text": "Project A yesterday",
            },
            {
                "conversation_id": "conv-b-today",
                "project_id": "project-b",
                "title": "B Today",
                "created_at": now,
                "updated_at": now,
                "message_count": 5,
                "file_path": "/test/b-today.jsonl",
                "full_text": "Project B today",
            },
        ]

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversations/all?project=project-a&date=today")

            assert response.status_code == 200
            data = response.json()

            # Should only have project-a from today
            assert data["total"] == 1
            assert data["results"][0]["conversation_id"] == "conv-a-today"


@pytest.mark.unit
class TestGetConversationEndpoint:
    """Tests for GET /api/conversation/{conversation_id} endpoint."""

    def test_get_conversation_success(self, client, mock_duckdb_store, tmp_path):
        """Test successfully retrieving a conversation."""
        # Create temporary JSONL file
        conv_file = tmp_path / "conv-1.jsonl"
        messages = [
            {"type": "user", "message": {"content": "Hello"}, "timestamp": "2025-01-01T10:00:00"},
            {"type": "assistant", "message": {"content": "Hi there!"}, "timestamp": "2025-01-01T10:00:05"}
        ]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        # Update mock meta to point to temp file
        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversation/conv-1")

            assert response.status_code == 200
            data = response.json()

            assert data["conversation_id"] == "conv-1"
            assert data["title"] == "Python Binary Search"
            assert data["message_count"] == 2
            assert len(data["messages"]) == 2
            assert data["messages"][0]["role"] == "user"
            assert data["messages"][0]["content"] == "Hello"
            assert data["messages"][1]["role"] == "assistant"
            assert data["messages"][1]["content"] == "Hi there!"

    def test_get_conversation_not_in_index(self, client, mock_duckdb_store):
        """Test error when conversation not found in index."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversation/nonexistent")

            assert response.status_code == 404
            assert "not found in index" in response.json()["detail"]

    def test_get_conversation_file_not_found(self, client, mock_duckdb_store):
        """Test error when conversation file doesn't exist."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            # conv-1 exists in index but file doesn't exist
            response = client.get("/api/conversation/conv-1")

            assert response.status_code == 404
            assert "file not found" in response.json()["detail"].lower()

    def test_get_conversation_invalid_json(self, client, mock_duckdb_store, tmp_path):
        """Test error handling for invalid JSON in conversation file."""
        conv_file = tmp_path / "invalid.jsonl"
        with open(conv_file, 'w') as f:
            f.write("invalid json\n")

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversation/conv-1")

            assert response.status_code == 500
            assert "invalid JSON" in response.json()["detail"]

    def test_get_conversation_with_list_content(self, client, mock_duckdb_store, tmp_path):
        """Test handling of content as list (text blocks)."""
        conv_file = tmp_path / "conv-list.jsonl"
        messages = [
            {
                "type": "user",
                "message": {
                    "content": [
                        {"type": "text", "text": "First block"},
                        {"type": "text", "text": "Second block"}
                    ]
                },
                "timestamp": "2025-01-01T10:00:00"
            }
        ]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversation/conv-1")

            assert response.status_code == 200
            data = response.json()

            # Content from list should be joined with newlines
            assert "First block\n\nSecond block" in data["messages"][0]["content"]

    def test_get_conversation_skips_non_user_assistant_messages(self, client, mock_duckdb_store, tmp_path):
        """Test that only user/assistant messages are included."""
        conv_file = tmp_path / "conv-mixed.jsonl"
        messages = [
            {"type": "user", "message": {"content": "User message"}, "timestamp": "2025-01-01T10:00:00"},
            {"type": "system", "message": {"content": "System message"}, "timestamp": "2025-01-01T10:00:01"},
            {"type": "assistant", "message": {"content": "Assistant message"}, "timestamp": "2025-01-01T10:00:02"}
        ]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            response = client.get("/api/conversation/conv-1")

            assert response.status_code == 200
            data = response.json()

            # Should only have 2 messages (user and assistant, not system)
            assert len(data["messages"]) == 2
            assert data["messages"][0]["role"] == "user"
            assert data["messages"][1]["role"] == "assistant"


@pytest.mark.unit
class TestResumeSessionEndpoint:
    """Tests for POST /api/resume endpoint."""

    def test_resume_claude_session_success(self, client, mock_duckdb_store, mock_platform_manager, tmp_path):
        """Test successfully resuming a Claude Code session."""
        # Create Claude JSONL file with cwd
        conv_file = tmp_path / "conv-1.jsonl"
        messages = [
            {"type": "user", "cwd": "/home/user/project", "message": {"content": "Test"}},
            {"type": "assistant", "message": {"content": "Response"}}
        ]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "conv-1"})

                assert response.status_code == 200
                data = response.json()

                assert data["success"] is True
                assert data["tool"] == "claude"
                assert data["cwd"] == "/home/user/project"
                assert "claude --resume conv-1" in data["command"]
                assert data["platform"] == "windows"

                # Verify terminal was opened
                mock_platform_manager.open_terminal_with_command.assert_called_once()

    def test_resume_vibe_session_success(self, client, mock_duckdb_store, mock_platform_manager, tmp_path):
        """Test successfully resuming a Vibe session."""
        # Create Vibe JSON file
        conv_file = tmp_path / "session_123.json"
        vibe_data = {
            "metadata": {
                "environment": {
                    "working_directory": "/home/user/vibe-project"
                }
            },
            "messages": []
        }
        with open(conv_file, 'w') as f:
            json.dump(vibe_data, f)

        now = datetime.now()
        mock_duckdb_store._data.append({
            "conversation_id": "session_123",
            "project_id": "vibe-project",
            "title": "Vibe Session",
            "created_at": now,
            "updated_at": now,
            "message_count": 5,
            "file_path": str(conv_file),
            "full_text": "Vibe conversation",
        })

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "session_123"})

                assert response.status_code == 200
                data = response.json()

                assert data["success"] is True
                assert data["tool"] == "vibe"
                assert data["cwd"] == "/home/user/vibe-project"
                assert "vibe --resume session_123" in data["command"]

    def test_resume_conversation_not_found(self, client, mock_duckdb_store, mock_platform_manager):
        """Test error when conversation doesn't exist."""
        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "nonexistent"})

                assert response.status_code == 404
                assert "not found" in response.json()["detail"]

    def test_resume_unknown_format(self, client, mock_duckdb_store, mock_platform_manager, tmp_path):
        """Test error for unknown conversation format."""
        # Create file with unknown extension
        conv_file = tmp_path / "conv.txt"
        conv_file.write_text("unknown format")

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "conv-1"})

                assert response.status_code == 400
                assert "Unknown conversation format" in response.json()["detail"]

    def test_resume_with_path_normalization(self, client, mock_duckdb_store, mock_platform_manager, tmp_path):
        """Test that paths are normalized for the platform."""
        conv_file = tmp_path / "conv-1.jsonl"
        messages = [{"type": "user", "cwd": "/mnt/c/Users/Test/project", "message": {"content": "Test"}}]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        # Mock normalize_path to convert WSL to Windows
        mock_platform_manager.normalize_path = Mock(return_value="C:\\Users\\Test\\project")

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "conv-1"})

                assert response.status_code == 200
                data = response.json()

                # Path should be normalized
                assert data["cwd"] == "C:\\Users\\Test\\project"
                mock_platform_manager.normalize_path.assert_called_once_with("/mnt/c/Users/Test/project")

    def test_resume_without_cwd(self, client, mock_duckdb_store, mock_platform_manager, tmp_path):
        """Test resuming when no cwd is found in conversation."""
        conv_file = tmp_path / "conv-1.jsonl"
        messages = [
            {"type": "user", "message": {"content": "Test"}},  # No cwd
            {"type": "assistant", "message": {"content": "Response"}}
        ]
        with open(conv_file, 'w') as f:
            for msg in messages:
                f.write(json.dumps(msg) + '\n')

        for row in mock_duckdb_store._data:
            if row["conversation_id"] == "conv-1":
                row["file_path"] = str(conv_file)

        with patch('searchat.api.routers.conversations.deps.get_duckdb_store', return_value=mock_duckdb_store):
            with patch('searchat.api.routers.conversations.get_platform_manager', return_value=mock_platform_manager):
                response = client.post("/api/resume", json={"conversation_id": "conv-1"})

                assert response.status_code == 200
                data = response.json()

                # cwd should be None
                assert data["cwd"] is None
                # Should still open terminal
                mock_platform_manager.open_terminal_with_command.assert_called_once()


def test_conversations_resolve_dataset_requires_snapshot() -> None:
    from searchat.api.routers import conversations as conv_router

    with pytest.raises(RuntimeError):
        conv_router._resolve_dataset(None)


def test_conversations_resolve_dataset_disabled_returns_404(monkeypatch: pytest.MonkeyPatch) -> None:
    from searchat.api.routers import conversations as conv_router

    config = type("Cfg", (), {"snapshots": type("S", (), {"enabled": False})()})()
    monkeypatch.setattr(conv_router.deps, "get_config", lambda: config)

    with pytest.raises(HTTPException) as excinfo:
        conv_router._resolve_dataset("backup_20250101_000000")
    assert excinfo.value.status_code == 404


def test_conversations_messages_from_parquet_parses_dict_and_tuple() -> None:
    from searchat.api.routers import conversations as conv_router

    now = datetime.now()
    messages = conv_router._messages_from_parquet(
        [
            {"role": "user", "content": "hi", "timestamp": now},
            (0, "assistant", "ok", "2025-01-01T00:00:00"),
        ]
    )
    assert len(messages) == 2
    assert messages[0].role == "user"
    assert messages[0].timestamp.startswith(str(now.date()))
    assert messages[1].role == "assistant"


def test_conversations_slice_messages_handles_invalid_range() -> None:
    from searchat.api.routers import conversations as conv_router
    from searchat.api.models import ConversationMessage

    msgs = [ConversationMessage(role="user", content="a", timestamp="")]
    assert conv_router._slice_messages(msgs, 2, 1) == []


def test_conversation_diff_no_similar_returns_404(client, monkeypatch: pytest.MonkeyPatch):
    from unittest.mock import AsyncMock
    from searchat.api.routers import conversations as conv_router

    monkeypatch.setattr(
        conv_router,
        "get_similar_conversations",
        AsyncMock(return_value={"similar_conversations": []}),
    )

    resp = client.get("/api/conversation/conv-1/diff")
    assert resp.status_code == 404
    assert resp.json()["detail"] == "No similar conversation found"


@pytest.mark.asyncio
async def test_conversation_diff_rejects_invalid_target_id_type() -> None:
    from searchat.api.routers import conversations as conv_router

    with pytest.raises(HTTPException) as excinfo:
        await conv_router.get_conversation_diff("conv-1", target_id=123)  # type: ignore[arg-type]
    assert excinfo.value.status_code == 400


@pytest.mark.asyncio
async def test_conversation_diff_computes_added_removed_counts(monkeypatch: pytest.MonkeyPatch) -> None:
    from searchat.api.routers import conversations as conv_router
    from searchat.api.models import ConversationMessage

    source = type(
        "C",
        (),
        {
            "messages": [
                ConversationMessage(role="user", content="a", timestamp=""),
                ConversationMessage(role="assistant", content="b", timestamp=""),
            ]
        },
    )()
    target = type(
        "C",
        (),
        {
            "messages": [
                ConversationMessage(role="user", content="a", timestamp=""),
                ConversationMessage(role="assistant", content="c", timestamp=""),
            ]
        },
    )()

    async def _fake_get_conversation(conversation_id: str, snapshot=None):
        return source if conversation_id == "conv-1" else target

    monkeypatch.setattr(conv_router, "get_conversation", _fake_get_conversation)

    payload = await conv_router.get_conversation_diff(
        "conv-1",
        target_id="conv-2",
        source_start=0,
        source_end=1,
    )
    assert payload["summary"]["added"] >= 1
    assert payload["summary"]["removed"] >= 1



"""Domain models for searchat - business logic data structures."""
from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime


@dataclass
class MessageRecord:
    """Record representing a single message in a conversation."""
    sequence: int
    role: str
    content: str
    timestamp: datetime
    has_code: bool
    code_blocks: list[str] = field(default_factory=list)


@dataclass
class ConversationRecord:
    """Record representing a full conversation with metadata."""
    conversation_id: str
    project_id: str
    file_path: str
    title: str
    created_at: datetime
    updated_at: datetime
    message_count: int
    messages: list[MessageRecord]
    full_text: str
    embedding_id: int
    file_hash: str
    indexed_at: datetime


@dataclass
class SearchFilters:
    """Filters for search queries."""
    project_ids: list[str] | None = None
    date_from: datetime | None = None
    date_to: datetime | None = None
    min_messages: int = 0
    has_code: bool | None = None
    tool: str | None = None


@dataclass
class SearchResult:
    """Single search result with metadata and score."""
    conversation_id: str
    project_id: str
    title: str
    created_at: datetime
    updated_at: datetime
    message_count: int
    file_path: str
    score: float
    snippet: str
    message_start_index: int | None = None
    message_end_index: int | None = None


@dataclass
class SearchResults:
    """Collection of search results with metadata."""
    results: list[SearchResult]
    total_count: int
    search_time_ms: float
    mode_used: str
    error: str | None = None


@dataclass
class IndexStats:
    """Statistics about the search index."""
    total_conversations: int
    total_messages: int
    index_time_seconds: float
    parquet_size_mb: float
    faiss_size_mb: float


@dataclass
class UpdateStats:
    """Statistics about an incremental index update."""
    new_conversations: int
    updated_conversations: int
    skipped_conversations: int
    update_time_seconds: float


@dataclass
class DateFilter:
    """Date range filter for search queries."""
    from_date: datetime | None
    to_date: datetime | None


@dataclass
class ParsedQuery:
    """Parsed search query with extracted components."""
    original: str
    must_include: list[str] = field(default_factory=list)
    should_include: list[str] = field(default_factory=list)
    must_exclude: list[str] = field(default_factory=list)
    exact_phrases: list[str] = field(default_factory=list)
    date_filter: DateFilter | None = None



"""Pytest configuration and fixtures for searchat tests.

CRITICAL: This file mocks heavy dependencies (sentence-transformers, faiss)
at import time to avoid TensorFlow/CUDA requirements during testing.
"""

from __future__ import annotations

import json
import sys
from unittest.mock import MagicMock

import numpy as np
import pytest

# Pre-import duckdb SQL types to avoid coverage import edge cases.


# ============================================================================
# IMPORT-TIME MOCKING (must happen before any searchat imports)
# ============================================================================

# Mock sentence-transformers to avoid tf_keras dependency
mock_st_module = MagicMock()
mock_st_class = MagicMock()
mock_st_instance = MagicMock()

# Configure mock embedding model behavior
def mock_encode(texts, **kwargs):
    """Return fake 384-dimensional embeddings."""
    if isinstance(texts, str):
        texts = [texts]
    return np.array([[0.1] * 384 for _ in texts])

mock_st_instance.encode = mock_encode
mock_st_instance.get_sentence_embedding_dimension.return_value = 384
mock_st_class.return_value = mock_st_instance
mock_st_module.SentenceTransformer = mock_st_class

# Inject mocked module into sys.modules BEFORE any imports
sys.modules['sentence_transformers'] = mock_st_module


# Mock FAISS to avoid GPU dependencies
class MockFaissIndex:
    """Mock FAISS index for testing."""

    def __init__(self, dimension: int = 384):
        self.d = dimension
        self.ntotal = 0
        self._vectors = []
        self._id_map = {}

    def add(self, vectors):
        """Mock adding vectors to index."""
        if len(vectors) > 0:
            self._vectors.extend(vectors)
            start = len(self._id_map)
            for idx, vector in enumerate(vectors):
                self._id_map[start + idx] = vector
            self.ntotal = len(self._id_map)

    def add_with_ids(self, vectors, ids):
        """Mock adding vectors with explicit IDs."""
        for vector, vector_id in zip(vectors, ids):
            self._id_map[int(vector_id)] = vector
        self.ntotal = len(self._id_map)

    def search(self, queries, k):
        """Mock search returning fake distances and indices."""
        n_queries = len(queries) if hasattr(queries, '__len__') else 1
        if n_queries == 0:
            return np.array([]), np.array([])
        # Return fake results
        distances = np.random.rand(n_queries, k).astype('float32')
        indices = np.random.randint(0, max(1, self.ntotal), (n_queries, k))
        return distances, indices

    def remove_ids(self, selector):
        """Mock removing vectors by IDs."""
        ids = getattr(selector, "ids", [])
        for vector_id in ids:
            self._id_map.pop(int(vector_id), None)
        self.ntotal = len(self._id_map)

    def reconstruct(self, vector_id):
        """Return stored vector by ID."""
        return self._id_map[int(vector_id)]

    def reset(self):
        """Reset the index."""
        self.ntotal = 0
        self._vectors = []
        self._id_map = {}


class MockFaissIndexIDMap2(MockFaissIndex):
    """Mock ID-mapped index wrapper."""


def mock_index_flat_l2(dimension):
    """Factory for creating mock FAISS index."""
    return MockFaissIndex(dimension)


def mock_index_idmap2(base_index):
    """Factory for creating mock IDMap2 index."""
    return MockFaissIndexIDMap2(base_index.d)


class MockIDSelectorBatch:
    def __init__(self, *args):
        if len(args) == 1:
            self.ids = list(args[0])
        elif len(args) == 2:
            count, ptr = args
            self.ids = list(np.ctypeslib.as_array(ptr, shape=(count,)))
        else:
            self.ids = []


mock_faiss_module = MagicMock()
mock_faiss_module.IndexFlatL2 = mock_index_flat_l2
mock_faiss_module.IndexIDMap2 = mock_index_idmap2
mock_faiss_module.IDSelectorBatch = MockIDSelectorBatch
mock_faiss_module._index_store = None


def mock_write_index(index, _path):
    mock_faiss_module._index_store = index


def mock_read_index(_path):
    if mock_faiss_module._index_store is None:
        mock_faiss_module._index_store = MockFaissIndex()
    return mock_faiss_module._index_store


mock_faiss_module.read_index = MagicMock(side_effect=mock_read_index)
mock_faiss_module.write_index = MagicMock(side_effect=mock_write_index)

sys.modules['faiss'] = mock_faiss_module


# NOTE: DuckDB mocking removed - analytics tests need the real module.
# Tests that need to mock duckdb should do so locally with pytest fixtures.


# ============================================================================
# PYTEST FIXTURES
# ============================================================================

# NOW safe to import searchat modules (after mocking)
from pathlib import Path
from datetime import datetime
from typing import Any

from searchat.core import ConversationIndexer
from searchat.services import BackupManager
from searchat.config import Config
from searchat.config import PathResolver


@pytest.fixture(autouse=True)
def _isolate_user_agent_dirs(monkeypatch):
    """Prevent tests from scanning real user home directories."""
    monkeypatch.setattr(PathResolver, "resolve_codex_dirs", staticmethod(lambda _cfg=None: []))
    monkeypatch.setattr(PathResolver, "resolve_gemini_dirs", staticmethod(lambda _cfg=None: []))


@pytest.fixture
def temp_data_dir(tmp_path):
    """Create temporary data directory structure for tests."""
    data_dir = tmp_path / "data"
    conversations_dir = data_dir / "conversations"
    indices_dir = data_dir / "indices"

    conversations_dir.mkdir(parents=True)
    indices_dir.mkdir(parents=True)

    return data_dir


@pytest.fixture
def temp_search_dir(tmp_path):
    """Create temporary search directory with full structure."""
    search_dir = tmp_path / ".searchat"
    data_dir = search_dir / "data"
    config_dir = search_dir / "config"
    backups_dir = search_dir / "backups"

    data_dir.mkdir(parents=True)
    config_dir.mkdir(parents=True)
    backups_dir.mkdir(parents=True)

    return search_dir


@pytest.fixture
def sample_claude_conversation(tmp_path) -> Path:
    """Create sample Claude Code conversation JSONL file."""
    conv_file = tmp_path / "test_conversation.jsonl"
    messages = [
        {
            "type": "user",
            "message": {"text": "How do I implement a binary search tree in Python?"},
            "timestamp": "2025-01-15T10:00:00",
            "uuid": "msg-1"
        },
        {
            "type": "assistant",
            "message": {
                "text": "Here's a binary search tree implementation:\n```python\nclass Node:\n    def __init__(self, value):\n        self.value = value\n        self.left = None\n        self.right = None\n```"
            },
            "timestamp": "2025-01-15T10:00:30",
            "uuid": "msg-2"
        },
        {
            "type": "user",
            "message": {"text": "Can you add insert and search methods?"},
            "timestamp": "2025-01-15T10:01:00",
            "uuid": "msg-3"
        },
    ]

    with open(conv_file, 'w', encoding='utf-8') as f:
        for msg in messages:
            f.write(json.dumps(msg) + '\n')

    return conv_file


@pytest.fixture
def sample_vibe_conversation(tmp_path) -> Path:
    """Create sample Mistral Vibe conversation JSON file."""
    conv_file = tmp_path / "session_20250115.json"
    data = {
        "id": "session-123",
        "created_at": "2025-01-15T10:00:00",
        "updated_at": "2025-01-15T10:05:00",
        "messages": [
            {
                "role": "user",
                "content": "Explain async/await in JavaScript",
                "timestamp": "2025-01-15T10:00:00"
            },
            {
                "role": "assistant",
                "content": "Async/await is syntactic sugar for promises in JavaScript. It allows you to write asynchronous code that looks synchronous.",
                "timestamp": "2025-01-15T10:00:15"
            },
            {
                "role": "user",
                "content": "Show me an example with error handling",
                "timestamp": "2025-01-15T10:02:00"
            },
        ]
    }

    with open(conv_file, 'w', encoding='utf-8') as f:
        json.dump(data, f, indent=2)

    return conv_file


@pytest.fixture
def indexer(temp_data_dir):
    """ConversationIndexer instance with temporary directory."""
    config = Config.load()
    return ConversationIndexer(temp_data_dir, config)


@pytest.fixture
def search_engine(temp_data_dir):
    """SearchEngine instance with temporary directory."""
    config = Config.load()
    from searchat.core.search_engine import SearchEngine

    return SearchEngine(temp_data_dir, config)


@pytest.fixture
def backup_manager(temp_search_dir):
    """BackupManager instance with temporary directory."""
    return BackupManager(temp_search_dir)


@pytest.fixture
def test_client():
    """FastAPI test client."""
    from fastapi.testclient import TestClient
    from searchat.web_api import app

    return TestClient(app)


@pytest.fixture
def mock_embeddings():
    """Mock sentence transformer embeddings for testing."""
    import numpy as np

    class MockEmbedder:
        def encode(self, texts, batch_size=32, show_progress_bar=False):
            """Return random embeddings for testing."""
            if isinstance(texts, str):
                texts = [texts]
            # Return consistent dimensions (384 for MiniLM)
            return np.random.rand(len(texts), 384).astype('float32')

    return MockEmbedder()


@pytest.fixture
def sample_config(tmp_path) -> dict[str, Any]:
    """Sample configuration for testing."""
    return {
        "embedding": {
            "model": "sentence-transformers/all-MiniLM-L6-v2",
            "dimension": 384,
            "batch_size": 32,
        },
        "search": {
            "default_mode": "hybrid",
            "max_results": 100,
            "min_score": 0.0,
        },
        "chunking": {
            "max_chunk_size": 1000,
            "overlap": 100,
        }
    }



import pytest
from pathlib import Path
from unittest.mock import Mock, patch


@pytest.mark.unit
def test_readiness_state_transitions():
    from searchat.api.readiness import get_readiness, warming_payload, error_payload

    readiness = get_readiness()

    # Warmup start is single-shot
    readiness.mark_warmup_started()
    assert readiness.mark_warmup_started() is False

    readiness.set_component("faiss", "loading")
    readiness.set_component("faiss", "error", error="boom")

    warm = warming_payload(retry_after_ms=123)
    assert warm["status"] == "warming"
    assert warm["retry_after_ms"] == 123
    assert warm["components"]["faiss"] == "error"
    assert warm["errors"]["faiss"] == "boom"

    err = error_payload()
    assert err["status"] == "error"
    assert err["errors"]["faiss"] == "boom"


@pytest.mark.unit
def test_invalidate_search_index_marks_semantic_stale():
    import searchat.api.dependencies as deps

    engine = Mock()
    with patch.object(deps, "start_background_warmup") as start_warmup:
        deps._search_engine = engine
        deps.projects_cache = ["x"]
        deps.stats_cache = {"y": 1}

        # Pretend semantic components were ready
        readiness = deps.get_readiness()
        readiness.set_component("faiss", "ready")
        readiness.set_component("metadata", "ready")
        readiness.set_component("embedder", "ready")

        deps.invalidate_search_index()

        assert deps.projects_cache is None
        assert deps.stats_cache is None
        engine.refresh_index.assert_called_once()
        start_warmup.assert_called_once()

        snap = readiness.snapshot()
        assert snap.components["faiss"] == "idle"
        assert snap.components["metadata"] == "idle"
        assert snap.components["embedder"] == "idle"


@pytest.mark.unit
def test_warmup_semantic_components_sets_readiness_ready(tmp_path: Path):
    import searchat.api.dependencies as deps

    # Ensure services look initialized for the warmup guard.
    deps._config = Mock()
    deps._search_dir = tmp_path

    engine = Mock()
    engine.ensure_metadata_ready = Mock()
    engine.ensure_faiss_loaded = Mock()
    engine.ensure_embedder_loaded = Mock()

    with patch.object(deps, "_ensure_search_engine", return_value=engine):
        readiness = deps.get_readiness()
        readiness.set_component("faiss", "idle")
        readiness.set_component("metadata", "idle")
        readiness.set_component("embedder", "idle")

        deps._warmup_semantic_components()

        engine.ensure_metadata_ready.assert_called_once()
        engine.ensure_faiss_loaded.assert_called_once()
        engine.ensure_embedder_loaded.assert_called_once()
        snap = readiness.snapshot()
        assert snap.components["faiss"] == "ready"
        assert snap.components["metadata"] == "ready"
        assert snap.components["embedder"] == "ready"


@pytest.mark.unit
def test_duckdb_store_queries_use_parameterized_parquet_scan(tmp_path: Path):
    from searchat.api.duckdb_store import DuckDBStore

    search_dir = tmp_path / ".searchat"
    conversations_dir = search_dir / "data" / "conversations"
    conversations_dir.mkdir(parents=True)
    (conversations_dir / "project_test.parquet").write_bytes(b"")

    store = DuckDBStore(search_dir, memory_limit_mb=64)

    con = Mock()
    cursor = Mock()
    con.execute.return_value = cursor
    cursor.fetchall.return_value = [("p1",), ("p2",)]

    with patch('duckdb.connect', return_value=con):
        projects = store.list_projects()

    assert projects == ["p1", "p2"]
    sql = con.execute.call_args[0][0]
    assert "parquet_scan(?)" in sql


@pytest.mark.unit
def test_duckdb_store_get_statistics_and_meta(tmp_path: Path):
    from searchat.api.duckdb_store import DuckDBStore

    search_dir = tmp_path / ".searchat"
    conversations_dir = search_dir / "data" / "conversations"
    conversations_dir.mkdir(parents=True)
    (conversations_dir / "project_test.parquet").write_bytes(b"")

    store = DuckDBStore(search_dir)

    con = Mock()
    cursor = Mock()
    con.execute.return_value = cursor
    cursor.fetchone.return_value = (1, 10, 10.0, 1, "2025-01-01", "2025-01-02")

    with patch('duckdb.connect', return_value=con):
        stats = store.get_statistics()

    assert stats.total_conversations == 1
    assert stats.total_messages == 10
    assert stats.total_projects == 1

    cursor.fetchone.return_value = (
        "conv-1",
        "proj",
        "Title",
        "2025-01-01",
        "2025-01-02",
        2,
        "/tmp/conv.jsonl",
    )

    with patch('duckdb.connect', return_value=con):
        meta = store.get_conversation_meta("conv-1")

    assert meta is not None
    assert meta["conversation_id"] == "conv-1"
    assert meta["file_path"] == "/tmp/conv.jsonl"



from __future__ import annotations

import json
from unittest.mock import Mock, patch

import pytest
from fastapi.testclient import TestClient

from searchat.api.app import app
from searchat.api.models.responses import ConversationMessage


@pytest.fixture
def client():
    return TestClient(app)


@pytest.fixture
def export_config_disabled():
    cfg = Mock()
    cfg.export = Mock(enable_ipynb=False, enable_pdf=False, enable_tech_docs=False)
    return cfg


@pytest.fixture
def export_config_enabled():
    cfg = Mock()
    cfg.export = Mock(enable_ipynb=True, enable_pdf=True, enable_tech_docs=False)
    return cfg


def test_export_ipynb_disabled_returns_404(client, export_config_disabled):
    mock_conv = Mock()
    mock_conv.conversation_id = "conv-123"
    mock_conv.title = "Test"
    mock_conv.project_id = "p"
    mock_conv.project_path = None
    mock_conv.tool = "claude"
    mock_conv.message_count = 1
    mock_conv.messages = [ConversationMessage(role="user", content="hi", timestamp="")] 

    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conv):
        with patch("searchat.api.routers.conversations.deps.get_config", return_value=export_config_disabled):
            resp = client.get("/api/conversation/conv-123/export?format=ipynb")

    assert resp.status_code == 404


def test_export_ipynb_enabled_returns_valid_notebook(client, export_config_enabled):
    mock_conv = Mock()
    mock_conv.conversation_id = "conv-123"
    mock_conv.title = "Notebook Test"
    mock_conv.project_id = "p"
    mock_conv.project_path = None
    mock_conv.tool = "claude"
    mock_conv.message_count = 1
    mock_conv.messages = [
        ConversationMessage(
            role="assistant",
            content="Text before\n```python\nprint('hi')\n```\nText after",
            timestamp="2026-01-28T10:00:00",
        )
    ]

    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conv):
        with patch("searchat.api.routers.conversations.deps.get_config", return_value=export_config_enabled):
            resp = client.get("/api/conversation/conv-123/export?format=ipynb")

    assert resp.status_code == 200
    assert "application/x-ipynb+json" in resp.headers["content-type"]

    nb = json.loads(resp.content)
    assert nb["nbformat"] == 4
    assert isinstance(nb["cells"], list)
    assert nb["cells"]


def test_export_pdf_disabled_returns_404(client, export_config_disabled):
    mock_conv = Mock()
    mock_conv.conversation_id = "conv-123"
    mock_conv.title = "Test"
    mock_conv.project_id = "p"
    mock_conv.project_path = None
    mock_conv.tool = "claude"
    mock_conv.message_count = 1
    mock_conv.messages = [ConversationMessage(role="user", content="hi", timestamp="")]

    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conv):
        with patch("searchat.api.routers.conversations.deps.get_config", return_value=export_config_disabled):
            resp = client.get("/api/conversation/conv-123/export?format=pdf")

    assert resp.status_code == 404


def test_export_pdf_enabled_returns_pdf_bytes(client, export_config_enabled):
    mock_conv = Mock()
    mock_conv.conversation_id = "conv-123"
    mock_conv.title = "PDF Test"
    mock_conv.project_id = "p"
    mock_conv.project_path = None
    mock_conv.tool = "claude"
    mock_conv.message_count = 1
    mock_conv.messages = [
        ConversationMessage(
            role="assistant",
            content="```python\ndef hello():\n    return 1\n```",
            timestamp="2026-01-28T10:00:00",
        )
    ]

    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conv):
        with patch("searchat.api.routers.conversations.deps.get_config", return_value=export_config_enabled):
            resp = client.get("/api/conversation/conv-123/export?format=pdf")

    assert resp.status_code == 200
    assert resp.headers["content-type"] == "application/pdf"
    assert resp.content.startswith(b"%PDF")


def test_bulk_export_ipynb_disabled_returns_404(client, export_config_disabled):
    with patch("searchat.api.routers.conversations.deps.get_config", return_value=export_config_disabled):
        resp = client.post(
            "/api/conversations/bulk-export",
            json={"conversation_ids": ["conv-1"], "format": "ipynb"},
        )
    assert resp.status_code == 404



from __future__ import annotations

from pathlib import Path

from fastapi.testclient import TestClient

from searchat.api.app import app


def test_index_html_contains_chat_controls():
    client = TestClient(app)
    resp = client.get("/")
    assert resp.status_code == 200

    html = resp.text
    assert "id=\"chatTemperature\"" in html
    assert "id=\"chatMaxTokens\"" in html
    assert "id=\"chatSystemPrompt\"" in html
    assert "id=\"chatSources\"" in html


def test_chat_js_targets_rag_endpoint_and_persists_options():
    chat_js = (
        Path(__file__).resolve().parents[2]
        / "src"
        / "searchat"
        / "web"
        / "static"
        / "js"
        / "modules"
        / "chat.js"
    )

    content = chat_js.read_text(encoding="utf-8")
    assert "/api/chat-rag" in content
    assert "chatTemperature" in content
    assert "chatMaxTokens" in content
    assert "chatSystemPrompt" in content
    assert "localStorage.setItem('chatTemperature'" in content
    assert "localStorage.setItem('chatMaxTokens'" in content
    assert "localStorage.setItem('chatSystemPrompt'" in content
    assert "event.key === 'Enter'" in content



"""Dashboards endpoints for widget-based saved query layouts."""
from __future__ import annotations

import json
from typing import Any

from fastapi import APIRouter, HTTPException
from fastapi.responses import JSONResponse, Response
from pydantic import BaseModel, Field

import searchat.api.dependencies as deps
from searchat.api.dependencies import (
    get_or_create_search_engine,
    get_search_engine,
    trigger_search_engine_warmup,
)
from searchat.api.models import SearchResultResponse
from searchat.api.readiness import error_payload, get_readiness, warming_payload
from searchat.api.utils import detect_source_from_path, detect_tool_from_path, parse_date_filter
from searchat.models import SearchFilters, SearchMode, SearchResult


router = APIRouter()


class DashboardWidgetRequest(BaseModel):
    id: str | None = None
    query_id: str
    title: str | None = None
    limit: int | None = Field(default=5, ge=1, le=100)
    sort_by: str | None = None
    layout: dict[str, int] | None = None


class DashboardLayoutRequest(BaseModel):
    widgets: list[DashboardWidgetRequest] = Field(min_length=1, max_length=50)
    columns: int | None = Field(default=None, ge=1, le=6)


class DashboardCreateRequest(BaseModel):
    name: str
    description: str | None = None
    layout: DashboardLayoutRequest
    queries: list[str] | None = None
    refresh_interval: int | None = Field(default=None, ge=1, le=86400)


class DashboardUpdateRequest(BaseModel):
    name: str | None = None
    description: str | None = None
    layout: DashboardLayoutRequest | None = None
    queries: list[str] | None = None
    refresh_interval: int | None = Field(default=None, ge=1, le=86400)


@router.get("/dashboards")
async def list_dashboards():
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        dashboards = service.list_dashboards()
        return {"total": len(dashboards), "dashboards": dashboards}
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.post("/dashboards")
async def create_dashboard(request: DashboardCreateRequest):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        dashboard = service.create_dashboard(request.model_dump())
        return {"success": True, "dashboard": dashboard}
    except ValueError as exc:
        raise HTTPException(status_code=400, detail=str(exc)) from exc
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.get("/dashboards/{dashboard_id}")
async def get_dashboard(dashboard_id: str):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        dashboard = service.get_dashboard(dashboard_id)
        if dashboard is None:
            raise HTTPException(status_code=404, detail="Dashboard not found")
        return {"dashboard": dashboard}
    except HTTPException:
        raise
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.put("/dashboards/{dashboard_id}")
async def update_dashboard(dashboard_id: str, request: DashboardUpdateRequest):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        updates = {k: v for k, v in request.model_dump().items() if v is not None}
        dashboard = service.update_dashboard(dashboard_id, updates)
        if dashboard is None:
            raise HTTPException(status_code=404, detail="Dashboard not found")
        return {"success": True, "dashboard": dashboard}
    except ValueError as exc:
        raise HTTPException(status_code=400, detail=str(exc)) from exc
    except HTTPException:
        raise
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.delete("/dashboards/{dashboard_id}")
async def delete_dashboard(dashboard_id: str):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        removed = service.delete_dashboard(dashboard_id)
        if not removed:
            raise HTTPException(status_code=404, detail="Dashboard not found")
        return {"success": True}
    except HTTPException:
        raise
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.get("/dashboards/{dashboard_id}/export")
async def export_dashboard(dashboard_id: str):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        service = deps.get_dashboards_service()
        dashboard = service.get_dashboard(dashboard_id)
        if dashboard is None:
            raise HTTPException(status_code=404, detail="Dashboard not found")
        content = json.dumps(dashboard, indent=2).encode("utf-8")
        return Response(
            content=content,
            media_type="application/json",
            headers={
                "Content-Disposition": f"attachment; filename=dashboard_{dashboard_id}.json",
            },
        )
    except HTTPException:
        raise
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


@router.get("/dashboards/{dashboard_id}/render")
async def render_dashboard(dashboard_id: str):
    config = deps.get_config()
    if not config.dashboards.enabled:
        raise HTTPException(status_code=404, detail="Dashboards are disabled")

    try:
        dashboards_service = deps.get_dashboards_service()
        saved_queries = deps.get_saved_queries_service()
        dashboard = dashboards_service.get_dashboard(dashboard_id)
        if dashboard is None:
            raise HTTPException(status_code=404, detail="Dashboard not found")

        widgets = _ensure_widgets(dashboard)
        widget_requests: list[dict[str, Any]] = []
        needs_semantic = False

        for widget in widgets:
            query_id = widget.get("query_id")
            query = saved_queries.get_query(query_id) if isinstance(query_id, str) else None
            if query is None:
                raise HTTPException(status_code=400, detail=f"Saved query {query_id} not found")

            query_text = query.get("query")
            if not isinstance(query_text, str):
                raise HTTPException(status_code=400, detail=f"Saved query {query_id} is invalid")

            mode = _parse_mode(query.get("mode"))
            if query_text.strip() == "*":
                mode = SearchMode.KEYWORD
            if mode in (SearchMode.HYBRID, SearchMode.SEMANTIC):
                needs_semantic = True

            widget_requests.append({"widget": widget, "query": query, "mode": mode})

        if needs_semantic:
            readiness = get_readiness().snapshot()
            for key in ("metadata", "faiss", "embedder"):
                if readiness.components.get(key) == "error":
                    return JSONResponse(status_code=500, content=error_payload())
            if any(readiness.components.get(key) != "ready" for key in ("metadata", "faiss", "embedder")):
                trigger_search_engine_warmup()
                return JSONResponse(status_code=503, content=warming_payload())
            search_engine = get_search_engine()
        else:
            search_engine = get_or_create_search_engine()

        rendered_widgets = []
        for request in widget_requests:
            widget = request["widget"]
            query = request["query"]
            mode = request["mode"]

            filters = _build_filters(query.get("filters"))
            results = search_engine.search(query["query"], mode=mode, filters=filters)
            sort_by = _normalize_sort_by(query.get("filters"))
            widget_sort = widget.get("sort_by")
            if isinstance(widget_sort, str) and widget_sort:
                sort_by = widget_sort

            sorted_results = _sort_results(results.results, sort_by)
            limit = widget.get("limit") if isinstance(widget.get("limit"), int) else 5
            trimmed = sorted_results[:limit]

            response_results = [
                SearchResultResponse(
                    conversation_id=r.conversation_id,
                    project_id=r.project_id,
                    title=r.title,
                    created_at=r.created_at.isoformat(),
                    updated_at=r.updated_at.isoformat(),
                    message_count=r.message_count,
                    file_path=r.file_path,
                    snippet=r.snippet,
                    score=r.score,
                    message_start_index=r.message_start_index,
                    message_end_index=r.message_end_index,
                    source=detect_source_from_path(r.file_path),
                    tool=detect_tool_from_path(r.file_path),
                )
                for r in trimmed
            ]

            rendered_widgets.append(
                {
                    "id": widget.get("id"),
                    "title": widget.get("title") or query.get("name") or "Saved Query",
                    "query_id": query.get("id"),
                    "query": query.get("query"),
                    "mode": mode.value,
                    "sort_by": sort_by,
                    "results": response_results,
                    "total": results.total_count,
                    "search_time_ms": results.search_time_ms,
                }
            )

        return {
            "dashboard": dashboard,
            "widgets": rendered_widgets,
        }
    except HTTPException:
        raise
    except ValueError as exc:
        raise HTTPException(status_code=400, detail=str(exc)) from exc
    except Exception as exc:
        raise HTTPException(status_code=500, detail=str(exc)) from exc


def _ensure_widgets(dashboard: dict[str, Any]) -> list[dict[str, Any]]:
    layout = dashboard.get("layout")
    if not isinstance(layout, dict):
        raise HTTPException(status_code=400, detail="Dashboard layout is invalid")
    widgets = layout.get("widgets")
    if not isinstance(widgets, list) or not widgets:
        raise HTTPException(status_code=400, detail="Dashboard layout is invalid")
    return widgets


def _parse_mode(mode_value: Any) -> SearchMode:
    mode_str = str(mode_value or "hybrid").lower()
    try:
        return SearchMode(mode_str)
    except Exception as exc:
        raise HTTPException(status_code=400, detail="Invalid search mode in saved query") from exc


def _build_filters(filters_value: Any) -> SearchFilters:
    filters = SearchFilters()
    if not isinstance(filters_value, dict):
        return filters

    project = filters_value.get("project")
    if isinstance(project, str) and project:
        filters.project_ids = [project]

    tool = filters_value.get("tool")
    if isinstance(tool, str) and tool:
        tool_value = tool.lower()
        if tool_value not in ("claude", "vibe", "opencode"):
            raise ValueError("Invalid tool filter in saved query")
        filters.tool = tool_value

    date = filters_value.get("date") if isinstance(filters_value.get("date"), str) else None
    date_from = filters_value.get("date_from") if isinstance(filters_value.get("date_from"), str) else None
    date_to = filters_value.get("date_to") if isinstance(filters_value.get("date_to"), str) else None
    filters.date_from, filters.date_to = parse_date_filter(date, date_from, date_to)

    return filters


def _normalize_sort_by(filters_value: Any) -> str:
    if isinstance(filters_value, dict):
        sort_by = filters_value.get("sort_by")
        if isinstance(sort_by, str) and sort_by:
            return sort_by
    return "relevance"


def _sort_results(results: list[SearchResult], sort_by: str) -> list[SearchResult]:
    sorted_results = results.copy()
    if sort_by == "date_newest":
        sorted_results.sort(key=lambda r: r.updated_at, reverse=True)
    elif sort_by == "date_oldest":
        sorted_results.sort(key=lambda r: r.updated_at, reverse=False)
    elif sort_by == "messages":
        sorted_results.sort(key=lambda r: r.message_count, reverse=True)
    return sorted_results



#!/usr/bin/env python
"""Simple CLI for Claude Search - no complex TUI, just clean terminal output"""
from __future__ import annotations

import sys
import os
from pathlib import Path
from datetime import datetime, timedelta
import json

from rich.console import Console
from rich.text import Text
from rich.panel import Panel
from rich.table import Table
from rich.prompt import Prompt, IntPrompt, Confirm
from rich.syntax import Syntax
from rich import print as rprint

from searchat.core import SearchEngine
from searchat.models import SearchMode, SearchFilters, SearchResult
from searchat.config import Config, PathResolver


class SearchCLI:
    def __init__(self):
        self.console = Console()
        self.config = Config.load()
        search_dir = PathResolver.get_shared_search_dir(self.config)
        self.search_engine = SearchEngine(search_dir, self.config)
        self.current_results: list[SearchResult] = []
        self.search_mode = SearchMode.HYBRID
        
    def run(self):
        """Main CLI loop"""
        self.console.clear()
        self.console.print("[bold cyan]Claude Conversation Search[/bold cyan]")
        self.console.print("Enter search query (or 'help' for commands, 'quit' to exit)\n")

        while True:
            try:
                # Get search query
                query = Prompt.ask("[bold green]Search[/bold green]")

                if query.lower() == 'quit':
                    break
                elif query.lower() == 'help':
                    self._show_help()
                    continue
                elif query.lower() == 'mode':
                    self._change_mode()
                    continue
                elif query.lower() == 'all':
                    self._show_all_conversations()
                    if self.current_results:
                        self._display_results()
                        # Ask to view a result
                        while True:
                            choice = Prompt.ask("\n[bold]Enter number to view (or 'b' to go back, 's' for new search)[/bold]")

                            if choice.lower() == 'b' or choice.lower() == 's':
                                break
                            elif choice.isdigit():
                                idx = int(choice) - 1
                                if 0 <= idx < len(self.current_results):
                                    self._view_conversation(self.current_results[idx])
                                    # After viewing, show results again
                                    self._display_results()
                                else:
                                    self.console.print("[red]Invalid selection[/red]")
                            else:
                                self.console.print("[red]Invalid input[/red]")
                    continue
                elif not query.strip():
                    continue

                # Execute search
                self._search(query)

                # Show results
                if self.current_results:
                    self._display_results()

                    # Ask to view a result
                    while True:
                        choice = Prompt.ask("\n[bold]Enter number to view (or 'b' to go back, 's' for new search)[/bold]")

                        if choice.lower() == 'b' or choice.lower() == 's':
                            break
                        elif choice.isdigit():
                            idx = int(choice) - 1
                            if 0 <= idx < len(self.current_results):
                                self._view_conversation(self.current_results[idx])
                                # After viewing, show results again
                                self._display_results()
                            else:
                                self.console.print("[red]Invalid selection[/red]")
                        else:
                            self.console.print("[red]Invalid input[/red]")

            except KeyboardInterrupt:
                self.console.print("\n[yellow]Search interrupted[/yellow]")
                continue
            except EOFError:
                break

        self.console.print("\n[green]Goodbye![/green]")
        
    def _show_help(self):
        """Show help information"""
        help_text = """
[bold cyan]Commands:[/bold cyan]
  [yellow]search query[/yellow] - Search for conversations
  [yellow]all[/yellow]          - Show all conversations (sorted by length)
  [yellow]mode[/yellow]         - Change search mode (hybrid/semantic/keyword)
  [yellow]help[/yellow]         - Show this help
  [yellow]quit[/yellow]         - Exit

[bold cyan]Search Tips:[/bold cyan]
  • Use specific terms like "batch 15" or "84%"
  • Combine concepts: "test cases openai batch"
  • Exact phrases: "truncation problem"
  • Mode affects results:
    - [green]Hybrid[/green]: Best overall (default)
    - [green]Semantic[/green]: Meaning-based
    - [green]Keyword[/green]: Exact matches
        """
        self.console.print(help_text)
        
    def _change_mode(self):
        """Change search mode"""
        modes = {
            "1": SearchMode.HYBRID,
            "2": SearchMode.SEMANTIC,
            "3": SearchMode.KEYWORD
        }
        
        self.console.print("\n[bold]Select search mode:[/bold]")
        self.console.print("1. Hybrid (default)")
        self.console.print("2. Semantic")
        self.console.print("3. Keyword")
        
        choice = Prompt.ask("Choice", choices=["1", "2", "3"])
        self.search_mode = modes[choice]
        self.console.print(f"[green]Mode changed to: {self.search_mode.value}[/green]")
        
    def _show_all_conversations(self):
        """Show all conversations sorted by message length"""
        self.console.print("\n[dim]Loading all conversations...[/dim]")

        try:
            # Get all conversations from search engine's dataframe
            df = self.search_engine.conversations_df.copy()

            # Filter out conversations with 0 messages
            df = df[df['message_count'] > 0]

            # Sort by message count (length) descending
            df = df.sort_values('message_count', ascending=False)

            # Convert to SearchResult objects (vectorized)
            self.current_results = [
                SearchResult(
                    conversation_id=row['conversation_id'],
                    project_id=row['project_id'],
                    title=row['title'],
                    created_at=row['created_at'],
                    updated_at=row['updated_at'],
                    message_count=row['message_count'],
                    file_path=row['file_path'],
                    score=0.0,
                    snippet=row['full_text'][:200] + ("..." if len(row['full_text']) > 200 else "")
                )
                for row in df.to_dict('records')
            ]

            self.console.print(
                f"[green]Showing all {len(self.current_results)} conversations[/green] "
                f"[dim](sorted by message length)[/dim]\n"
            )
        except Exception as e:
            self.console.print(f"[red]Error loading conversations: {e}[/red]")
            self.current_results = []

    def _search(self, query: str):
        """Execute search"""
        self.console.print(f"\n[dim]Searching in {self.search_mode.value} mode...[/dim]")

        try:
            results = self.search_engine.search(query, mode=self.search_mode)
            self.current_results = results.results

            self.console.print(
                f"[green]Found {results.total_count} results[/green] "
                f"[dim]({results.search_time_ms:.0f}ms)[/dim]\n"
            )
        except Exception as e:
            self.console.print(f"[red]Search error: {e}[/red]")
            self.current_results = []
            
    def _display_results(self):
        """Display search results with context"""
        if not self.current_results:
            self.console.print("[yellow]No results found[/yellow]")
            return
            
        # Show up to 20 results
        for i, result in enumerate(self.current_results[:20], 1):
            # Build result display
            text = Text()
            
            # Number and source
            text.append(f"{i:2}. ", style="bold cyan")
            source = "WSL" if "/home/" in result.file_path else "WIN"
            text.append(f"[{source}] ", style="magenta" if source == "WSL" else "blue")
            
            # Date and project
            text.append(result.updated_at.strftime("%b %d %H:%M"), style="dim")
            text.append(f" | {result.project_id[:30]}\n", style="dim")
            
            # Title
            text.append(f"    {result.title[:70]}", style="bold")
            text.append(f" ({result.message_count} msgs)", style="dim")
            
            # Message range if available
            if result.message_start_index is not None:
                text.append(f" [msgs {result.message_start_index}-{result.message_end_index}]", style="yellow dim")
            text.append("\n")
            
            # Snippet with highlighting
            snippet = Text(f"    {result.snippet[:150]}...")
            # Highlight search terms (simple approach)
            query_words = self.current_results[0].snippet[:0]  # Get query from first result
            snippet.stylize("italic dim")
            
            text.append(snippet)
            
            # Display with separator
            self.console.print(text)
            self.console.print("    " + "─" * 70, style="dim")
            
        if len(self.current_results) > 20:
            self.console.print(f"\n[dim]Showing first 20 of {len(self.current_results)} results[/dim]")
            
    def _view_conversation(self, result: SearchResult):
        """View a conversation with pagination"""
        self.console.clear()
        
        # Header
        header = Panel(
            f"[bold]{result.title}[/bold]\n"
            f"Project: {result.project_id} | "
            f"Messages: {result.message_count} | "
            f"Updated: {result.updated_at.strftime('%Y-%m-%d %H:%M')}",
            title="Conversation",
            border_style="cyan"
        )
        self.console.print(header)
        
        # Load conversation
        try:
            with open(result.file_path, 'r', encoding='utf-8') as f:
                lines = [json.loads(line) for line in f]
                
            messages = []
            for entry in lines:
                if entry.get('type') in ('user', 'assistant'):
                    raw_content = entry.get('message', {}).get('content', '')
                    if isinstance(raw_content, str):
                        content = raw_content
                    elif isinstance(raw_content, list):
                        content = '\n\n'.join(
                            block.get('text', '') 
                            for block in raw_content 
                            if block.get('type') == 'text'
                        )
                    else:
                        content = ''
                        
                    if content:
                        messages.append({
                            'role': entry.get('type'),
                            'content': content,
                            'timestamp': entry.get('timestamp', '')
                        })
                        
            # Pagination
            page_size = 10
            start_idx = result.message_start_index or 0
            current_page = start_idx // page_size
            
            while True:
                self.console.clear()
                self.console.print(header)
                
                # Calculate page range
                start = current_page * page_size
                end = min(start + page_size, len(messages))
                
                self.console.print(f"\n[dim]Messages {start+1}-{end} of {len(messages)}[/dim]\n")
                
                # Display messages
                for i in range(start, end):
                    msg = messages[i]
                    
                    # Role header
                    role_color = "cyan" if msg['role'] == 'user' else "green"
                    self.console.print(f"[bold {role_color}][{msg['role'].upper()}] Message {i+1}[/bold {role_color}]")
                    
                    # Content (truncated)
                    content = msg['content'][:1000]
                    if len(msg['content']) > 1000:
                        content += "...[truncated]"
                    self.console.print(content)
                    self.console.print("─" * 80, style="dim")
                    
                # Navigation
                nav_text = "\n[bold]Navigation:[/bold] "
                if current_page > 0:
                    nav_text += "[p]revious page | "
                if end < len(messages):
                    nav_text += "[n]ext page | "
                nav_text += "[j]ump to message | [b]ack to results"
                
                self.console.print(nav_text)
                choice = Prompt.ask("Choice", default="b")
                
                if choice.lower() == 'b':
                    break
                elif choice.lower() == 'n' and end < len(messages):
                    current_page += 1
                elif choice.lower() == 'p' and current_page > 0:
                    current_page -= 1
                elif choice.lower() == 'j':
                    msg_num = IntPrompt.ask("Jump to message number", default=1)
                    if 1 <= msg_num <= len(messages):
                        current_page = (msg_num - 1) // page_size
                    else:
                        self.console.print("[red]Invalid message number[/red]")
                        
        except Exception as e:
            self.console.print(f"[red]Error loading conversation: {e}[/red]")
            Prompt.ask("\n[Press Enter to continue]")
            
        self.console.clear()


def main():
    """Entry point"""
    prog = Path(sys.argv[0]).name
    argv = set(sys.argv[1:])
    if prog.startswith("searchat"):
        if "--version" in argv:
            from searchat import __version__

            print(__version__)
            return
        if "-h" in argv or "--help" in argv:
            print("Usage: searchat")
            print()
            print("Interactive terminal UI for searching indexed conversations.")
            print()
            return

    cli = SearchCLI()
    cli.run()


if __name__ == "__main__":
    main()



"""Unit tests for statistics and backup API routes."""
import pytest
from datetime import datetime
from unittest.mock import Mock, patch
from pathlib import Path

from fastapi.testclient import TestClient

from searchat.api.app import app


@pytest.fixture
def client():
    """FastAPI test client."""
    return TestClient(app)


@pytest.fixture
def mock_duckdb_store_stats():
    """Mock DuckDBStore for stats endpoint."""
    from searchat.api.duckdb_store import IndexStatistics

    mock = Mock()
    mock.get_statistics.return_value = IndexStatistics(
        total_conversations=3,
        total_messages=30,
        avg_messages=10.0,
        total_projects=2,
        earliest_date="2025-01-01T00:00:00",
        latest_date="2025-01-20T00:00:00",
    )
    return mock


@pytest.fixture
def mock_backup_manager():
    """Mock BackupManager."""
    mock = Mock()
    mock.backup_dir = Path("/backups")

    # Mock BackupMetadata
    mock_metadata = Mock()
    mock_metadata.to_dict.return_value = {
        "backup_path": "/backups/backup_20250120_100000",
        "timestamp": "20250120_100000",
        "file_count": 5,
        "total_size_mb": 10.5
    }

    mock.create_backup.return_value = mock_metadata
    mock.list_backups.return_value = [mock_metadata]
    mock.restore_from_backup.return_value = None
    mock.delete_backup.return_value = None

    return mock


# ============================================================================
# STATISTICS ENDPOINT TESTS
# ============================================================================

@pytest.mark.unit
class TestStatisticsEndpoint:
    """Tests for GET /api/statistics endpoint."""

    def test_get_statistics_success(self, client, mock_duckdb_store_stats):
        """Test getting index statistics."""
        with patch('searchat.api.routers.stats.deps.get_duckdb_store', return_value=mock_duckdb_store_stats):
            with patch('searchat.api.routers.stats.deps.stats_cache', None):
                response = client.get("/api/statistics")

            assert response.status_code == 200
            data = response.json()

            assert "total_conversations" in data
            assert "total_messages" in data
            assert "avg_messages" in data
            assert "total_projects" in data
            assert "earliest_date" in data
            assert "latest_date" in data

            # Verify values
            assert data["total_conversations"] == 3
            assert data["total_messages"] == 30  # 10 + 5 + 15
            assert data["avg_messages"] == 10.0  # 30 / 3
            assert data["total_projects"] == 2  # project-a, project-b
            assert data["earliest_date"] == "2025-01-01T00:00:00"
            assert data["latest_date"] == "2025-01-20T00:00:00"

    def test_get_statistics_single_conversation(self, client):
        """Test statistics with single conversation."""
        from searchat.api.duckdb_store import IndexStatistics

        mock_store = Mock()
        now = datetime.now().isoformat()
        mock_store.get_statistics.return_value = IndexStatistics(
            total_conversations=1,
            total_messages=10,
            avg_messages=10.0,
            total_projects=1,
            earliest_date=now,
            latest_date=now,
        )

        with patch('searchat.api.routers.stats.deps.get_duckdb_store', return_value=mock_store):
            with patch('searchat.api.routers.stats.deps.stats_cache', None):
                response = client.get("/api/statistics")

            assert response.status_code == 200
            data = response.json()

            assert data["total_conversations"] == 1
            assert data["total_messages"] == 10
            assert data["avg_messages"] == 10.0
            assert data["total_projects"] == 1

    def test_get_statistics_uses_cache(self, client):
        """Second call should return cached payload for active dataset."""
        from searchat.api.duckdb_store import IndexStatistics

        mock_store = Mock()
        mock_store.get_statistics.return_value = IndexStatistics(
            total_conversations=2,
            total_messages=20,
            avg_messages=10.0,
            total_projects=1,
            earliest_date="2025-01-01T00:00:00",
            latest_date="2025-01-02T00:00:00",
        )

        with patch('searchat.api.routers.stats.deps.get_duckdb_store', return_value=mock_store):
            with patch('searchat.api.routers.stats.deps.stats_cache', None):
                first = client.get("/api/statistics")
                second = client.get("/api/statistics")

        assert first.status_code == 200
        assert second.status_code == 200
        assert mock_store.get_statistics.call_count == 1

    def test_get_statistics_snapshot_uses_dataset_store(self, client, mock_duckdb_store_stats):
        """Snapshot requests should bypass stats_cache and use get_duckdb_store_for."""
        with patch('searchat.api.routers.stats.deps.resolve_dataset_search_dir', return_value=(Path('/tmp/snap'), 'snap')):
            with patch('searchat.api.routers.stats.deps.get_duckdb_store_for', return_value=mock_duckdb_store_stats) as get_store:
                resp = client.get("/api/statistics?snapshot=backup_20250101_000000")

        assert resp.status_code == 200
        get_store.assert_called_once()

    def test_get_statistics_snapshot_not_found_returns_404(self, client):
        with patch('searchat.api.routers.stats.deps.resolve_dataset_search_dir', side_effect=ValueError("Snapshot not found")):
            resp = client.get("/api/statistics?snapshot=missing")
        assert resp.status_code == 404
        assert resp.json()["detail"] == "Snapshot not found"

    def test_get_statistics_snapshot_invalid_returns_400(self, client):
        with patch('searchat.api.routers.stats.deps.resolve_dataset_search_dir', side_effect=ValueError("Invalid snapshot name")):
            resp = client.get("/api/statistics?snapshot=../nope")
        assert resp.status_code == 400
        assert resp.json()["detail"] == "Invalid snapshot name"


# ============================================================================
# BACKUP ENDPOINT TESTS
# ============================================================================

@pytest.mark.unit
class TestCreateBackupEndpoint:
    """Tests for POST /api/backup/create endpoint."""

    def test_create_backup_success(self, client, mock_backup_manager):
        """Test creating a backup successfully."""
        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.post("/api/backup/create")

            assert response.status_code == 200
            data = response.json()

            assert data["success"] is True
            assert "backup" in data
            assert "message" in data
            assert "Backup created" in data["message"]

            mock_backup_manager.create_backup.assert_called_once_with(backup_name=None)

    def test_create_backup_with_name(self, client, mock_backup_manager):
        """Test creating a backup with custom name."""
        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.post("/api/backup/create?backup_name=my_backup")

            assert response.status_code == 200
            mock_backup_manager.create_backup.assert_called_once_with(backup_name="my_backup")

    def test_create_backup_error(self, client, mock_backup_manager):
        """Test error handling when backup fails."""
        mock_backup_manager.create_backup.side_effect = Exception("Disk full")

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.post("/api/backup/create")

            assert response.status_code == 500
            assert "Disk full" in response.json()["detail"]


@pytest.mark.unit
class TestListBackupsEndpoint:
    """Tests for GET /api/backup/list endpoint."""

    def test_list_backups_success(self, client, mock_backup_manager):
        """Test listing all backups."""
        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.get("/api/backup/list")

            assert response.status_code == 200
            data = response.json()

            assert "backups" in data
            assert "total" in data
            assert "backup_directory" in data
            assert data["total"] == 1
            assert len(data["backups"]) == 1

    def test_list_backups_empty(self, client, mock_backup_manager):
        """Test listing when no backups exist."""
        mock_backup_manager.list_backups.return_value = []

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.get("/api/backup/list")

            assert response.status_code == 200
            data = response.json()

            assert data["total"] == 0
            assert len(data["backups"]) == 0

    def test_list_backups_error(self, client, mock_backup_manager):
        """Test error handling when listing fails."""
        mock_backup_manager.list_backups.side_effect = Exception("Permission denied")

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.get("/api/backup/list")

            assert response.status_code == 500
            assert "Permission denied" in response.json()["detail"]


@pytest.mark.unit
class TestRestoreBackupEndpoint:
    """Tests for POST /api/backup/restore endpoint."""

    def test_restore_backup_success(self, client, mock_backup_manager, tmp_path):
        """Test restoring from a backup."""
        # Create backup directory
        backup_dir = tmp_path / "backup_20250120_100000"
        backup_dir.mkdir()

        mock_backup_manager.backup_dir = tmp_path
        mock_backup_manager.restore_from_backup.return_value = None

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            with patch('searchat.api.routers.backup.invalidate_search_index') as invalidate:
                response = client.post("/api/backup/restore?backup_name=backup_20250120_100000")

                assert response.status_code == 200
                data = response.json()

                assert data["success"] is True
                assert data["restored_from"] == "backup_20250120_100000"
                assert "Successfully restored" in data["message"]

                mock_backup_manager.restore_from_backup.assert_called_once()
                invalidate.assert_called_once()

    def test_restore_backup_not_found(self, client, mock_backup_manager, tmp_path):
        """Test error when backup doesn't exist."""
        mock_backup_manager.backup_dir = tmp_path

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.post("/api/backup/restore?backup_name=nonexistent")

            assert response.status_code == 404
            assert "not found" in response.json()["detail"]

    def test_restore_backup_with_pre_restore_backup(self, client, mock_backup_manager, tmp_path):
        """Test restore creates pre-restore backup."""
        backup_dir = tmp_path / "backup_20250120_100000"
        backup_dir.mkdir()

        mock_backup_manager.backup_dir = tmp_path

        # Mock pre-restore backup metadata
        pre_restore_meta = Mock()
        pre_restore_meta.to_dict.return_value = {"backup_path": "/backups/pre_restore"}
        mock_backup_manager.restore_from_backup.return_value = pre_restore_meta

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            with patch('searchat.api.routers.backup.invalidate_search_index'):
                response = client.post("/api/backup/restore?backup_name=backup_20250120_100000")

                assert response.status_code == 200
                data = response.json()

                assert "pre_restore_backup" in data
                assert data["pre_restore_backup"]["backup_path"] == "/backups/pre_restore"


@pytest.mark.unit
class TestDeleteBackupEndpoint:
    """Tests for DELETE /api/backup/delete/{backup_name} endpoint."""

    def test_delete_backup_success(self, client, mock_backup_manager, tmp_path):
        """Test deleting a backup."""
        backup_dir = tmp_path / "backup_20250120_100000"
        backup_dir.mkdir()

        mock_backup_manager.backup_dir = tmp_path

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.delete("/api/backup/delete/backup_20250120_100000")

            assert response.status_code == 200
            data = response.json()

            assert data["success"] is True
            assert data["deleted"] == "backup_20250120_100000"
            assert "Backup deleted" in data["message"]

            mock_backup_manager.delete_backup.assert_called_once_with(backup_dir)

    def test_delete_backup_not_found(self, client, mock_backup_manager, tmp_path):
        """Test error when backup doesn't exist."""
        mock_backup_manager.backup_dir = tmp_path

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.delete("/api/backup/delete/nonexistent")

            assert response.status_code == 404
            assert "not found" in response.json()["detail"]

    def test_delete_backup_error(self, client, mock_backup_manager, tmp_path):
        """Test error handling when deletion fails."""
        backup_dir = tmp_path / "backup_20250120_100000"
        backup_dir.mkdir()

        mock_backup_manager.backup_dir = tmp_path
        mock_backup_manager.delete_backup.side_effect = Exception("Permission denied")

        with patch('searchat.api.routers.backup.get_backup_manager', return_value=mock_backup_manager):
            response = client.delete("/api/backup/delete/backup_20250120_100000")

            assert response.status_code == 500
            assert "Permission denied" in response.json()["detail"]



from __future__ import annotations

import re
from datetime import datetime, timedelta
from searchat.models import ParsedQuery, DateFilter


class QueryParser:
    def parse(self, query: str) -> ParsedQuery:
        result = ParsedQuery(original=query)
        
        # Extract phrases in either single or double quotes
        result.exact_phrases = re.findall(r'"([^"]+)"|\'([^\']+)\'', query)
        # Flatten the tuples and remove empty strings
        result.exact_phrases = [phrase for pair in result.exact_phrases for phrase in pair if phrase]
        # Remove both types of quoted phrases from query
        query = re.sub(r'"[^"]+"', '', query)
        query = re.sub(r"'[^']+'", '', query)
        
        result.must_include = re.findall(r'\+(\w+)', query)
        query = re.sub(r'\+\w+', '', query)
        
        result.must_exclude = re.findall(r'-(\w+)', query)
        query = re.sub(r'-\w+', '', query)
        
        result.date_filter = self._extract_date_filter(query)
        query = self._remove_date_terms(query)
        
        if ' AND ' in query.upper():
            result.must_include.extend([
                term.strip() 
                for term in re.split(r'\s+AND\s+', query, flags=re.IGNORECASE)
                if term.strip()
            ])
        elif ' OR ' in query.upper():
            result.should_include = [
                term.strip() 
                for term in re.split(r'\s+OR\s+', query, flags=re.IGNORECASE)
                if term.strip()
            ]
        else:
            result.should_include = [term for term in query.split() if term.strip()]
        
        return result
    
    def _extract_date_filter(self, query: str) -> DateFilter | None:
        query_lower = query.lower()
        now = datetime.now()
        
        if 'today' in query_lower:
            return DateFilter(from_date=now.replace(hour=0, minute=0, second=0, microsecond=0), to_date=now)
        elif 'last week' in query_lower or 'last 7 days' in query_lower:
            return DateFilter(from_date=now - timedelta(days=7), to_date=now)
        elif 'last 30 days' in query_lower or 'last month' in query_lower:
            return DateFilter(from_date=now - timedelta(days=30), to_date=now)
        elif 'last 3 months' in query_lower:
            return DateFilter(from_date=now - timedelta(days=90), to_date=now)
        
        return None
    
    def _remove_date_terms(self, query: str) -> str:
        date_terms = [
            'today', 'last week', 'last 7 days', 'last 30 days', 
            'last month', 'last 3 months'
        ]
        
        result = query
        for term in date_terms:
            result = re.sub(rf'\b{term}\b', '', result, flags=re.IGNORECASE)
        
        return ' '.join(result.split())



"""
Interactive setup wizard for Claude Search.

Run with: python -m searchat.setup
"""

import os
import sys
import shutil
from pathlib import Path
from typing import Optional, List

from searchat.config.constants import (
    DEFAULT_DATA_DIR,
    DEFAULT_CONFIG_SUBDIR,
    DEFAULT_DATA_SUBDIR,
    DEFAULT_LOGS_SUBDIR,
    SETTINGS_FILE,
    DEFAULT_SETTINGS_FILE,
    ENV_FILE,
    CLAUDE_DIR_NAME,
    CLAUDE_PROJECTS_SUBDIR,
)
from searchat.config import PathResolver


class SetupManager:
    """Manages interactive setup for first-time users."""

    def __init__(self):
        self.platform = PathResolver.detect_platform()
        self.claude_dirs: List[Path] = []
        self.data_dir = DEFAULT_DATA_DIR
        self.config_dir = self.data_dir / DEFAULT_CONFIG_SUBDIR

    def run_interactive_setup(self) -> bool:
        """
        Run the interactive setup wizard.

        Returns:
            True if setup completed successfully
        """
        print("=" * 70)
        print("Claude Search - Interactive Setup")
        print("=" * 70)
        print()

        # Step 1: Detect platform
        self._show_platform_info()

        # Step 2: Find Claude directories
        if not self._detect_claude_directories():
            return False

        # Step 3: Create directory structure
        if not self._create_directory_structure():
            return False

        # Step 4: Create configuration files
        if not self._create_config_files():
            return False

        # Step 5: Show next steps
        self._show_completion_message()

        return True

    def _show_platform_info(self):
        """Display detected platform information."""
        print(f"Detected platform: {self.platform}")
        print()

        platform_notes = {
            "windows": "Running on Windows. Will search for both Windows and WSL Claude directories.",
            "wsl": "Running under WSL. Will search for Linux paths and Windows mount points.",
            "linux": "Running on Linux. Will search for standard Linux paths.",
            "macos": "Running on macOS. Will search for standard macOS paths.",
        }

        note = platform_notes.get(self.platform, "Unknown platform")
        print(f"Note: {note}")
        print()

    def _detect_claude_directories(self) -> bool:
        """
        Attempt to auto-detect Claude conversation directories.

        Returns:
            True if at least one directory found
        """
        print("Searching for Claude conversation directories...")
        print()

        # Try common locations
        candidates = [
            Path.home() / CLAUDE_DIR_NAME / CLAUDE_PROJECTS_SUBDIR,
            Path.home() / CLAUDE_DIR_NAME,
        ]

        # Platform-specific additional candidates
        if self.platform == "windows":
            # Check for WSL paths
            wsl_paths = [
                Path("\\\\wsl$\\Ubuntu\\home") / os.getenv("USER", "user") / CLAUDE_DIR_NAME,
                Path("\\\\wsl.localhost\\Ubuntu\\home") / os.getenv("USER", "user") / CLAUDE_DIR_NAME,
            ]
            candidates.extend(wsl_paths)

        elif self.platform in ("wsl", "linux"):
            # Check Windows mount points
            username = os.getenv("USER", "user")
            mount_paths = [
                Path(f"/mnt/c/Users/{username}") / CLAUDE_DIR_NAME,
            ]
            candidates.extend(mount_paths)

        # Find existing directories
        found_dirs = []
        for candidate in candidates:
            if candidate.exists() and candidate.is_dir():
                found_dirs.append(candidate)
                print(f"  ✓ Found: {candidate}")

        if not found_dirs:
            print("  ✗ No Claude directories found automatically")
            print()
            print("Please ensure Claude CLI is installed and has created conversations.")
            print()
            response = input("Would you like to specify a custom path? (y/n): ").strip().lower()

            if response == "y":
                custom_path = input("Enter path to Claude directory: ").strip()
                custom_path_obj = Path(custom_path)
                if custom_path_obj.exists():
                    found_dirs.append(custom_path_obj)
                    print(f"  ✓ Using custom path: {custom_path_obj}")
                else:
                    print(f"  ✗ Path not found: {custom_path_obj}")
                    return False
            else:
                print()
                print("Setup cannot continue without a Claude directory.")
                print("Please install Claude CLI and create at least one conversation.")
                return False

        self.claude_dirs = found_dirs
        print()
        return True

    def _create_directory_structure(self) -> bool:
        """
        Create necessary directory structure.

        Returns:
            True if successful
        """
        print("Creating directory structure...")
        print()

        directories = [
            self.data_dir,
            self.config_dir,
            self.data_dir / DEFAULT_DATA_SUBDIR,
            self.data_dir / DEFAULT_LOGS_SUBDIR,
        ]

        for directory in directories:
            try:
                directory.mkdir(parents=True, exist_ok=True)
                print(f"  ✓ Created: {directory}")
            except PermissionError:
                print(f"  ✗ Permission denied: {directory}")
                return False
            except Exception as e:
                print(f"  ✗ Error creating {directory}: {e}")
                return False

        print()
        return True

    def _create_config_files(self) -> bool:
        """
        Create initial configuration files.

        Returns:
            True if successful
        """
        print("Creating configuration files...")
        print()

        # 1. Copy default settings to user config
        user_config = self.config_dir / SETTINGS_FILE
        default_config = Path(__file__).parent.parent / "config" / DEFAULT_SETTINGS_FILE

        if user_config.exists():
            response = input(f"Config file already exists at {user_config}. Overwrite? (y/n): ").strip().lower()
            if response != "y":
                print("  - Skipped settings.toml (already exists)")
                return True

        try:
            if default_config.exists():
                shutil.copy(default_config, user_config)
                print(f"  ✓ Created: {user_config}")

                # Update with detected paths
                self._update_config_paths(user_config)
            else:
                print(f"  ✗ Default config not found: {default_config}")
                return False
        except Exception as e:
            print(f"  ✗ Error creating config: {e}")
            return False

        # 2. Create .env file if it doesn't exist
        env_file = self.config_dir / ENV_FILE
        env_example = Path(__file__).parent.parent / ".env.example"

        if not env_file.exists():
            try:
                if env_example.exists():
                    shutil.copy(env_example, env_file)
                    print(f"  ✓ Created: {env_file}")
                else:
                    # Create minimal .env
                    env_file.write_text(
                        "# Searchat Environment Variables\n"
                        "# Uncomment and modify as needed\n\n"
                        f"# SEARCHAT_DATA_DIR={self.data_dir}\n"
                        "# SEARCHAT_PORT=8000\n"
                    )
                    print(f"  ✓ Created: {env_file}")
            except Exception as e:
                print(f"  ✗ Error creating .env: {e}")
                # Non-fatal, continue

        print()
        return True

    def _update_config_paths(self, config_path: Path):
        """Update config file with detected paths."""
        try:
            content = config_path.read_text()

            # Replace {username} placeholder
            username = os.getenv("USERNAME") or os.getenv("USER") or "user"
            content = content.replace("{username}", username)

            # Update search directory to match data_dir
            content = content.replace(
                'search_directory = "C:/Users/{username}/.searchat"',
                f'search_directory = "{str(self.data_dir).replace(chr(92), "/")}"'
            )

            config_path.write_text(content)
            print(f"  ✓ Updated config with your username: {username}")
        except Exception as e:
            print(f"  ⚠ Warning: Could not update config paths: {e}")

    def _show_completion_message(self):
        """Display setup completion message and next steps."""
        print("=" * 70)
        print("Setup Complete!")
        print("=" * 70)
        print()
        print("Configuration files created:")
        print(f"  - Config: {self.config_dir / SETTINGS_FILE}")
        print(f"  - Env:    {self.config_dir / ENV_FILE}")
        print()
        print("Data directory:")
        print(f"  - {self.data_dir}")
        print()
        print("Next steps:")
        print()
        print("1. Start the web interface:")
        print("   python csw.py")
        print()
        print("2. Or use the CLI:")
        print("   python cs.py \"your search query\"")
        print()
        print("3. Customize configuration:")
        print(f"   Edit {self.config_dir / SETTINGS_FILE}")
        print(f"   Edit {self.config_dir / ENV_FILE}")
        print()
        print("For more information, see README.md")
        print()


def main():
    """Entry point for setup wizard."""
    setup = SetupManager()

    try:
        success = setup.run_interactive_setup()
        sys.exit(0 if success else 1)
    except KeyboardInterrupt:
        print()
        print()
        print("Setup cancelled by user.")
        sys.exit(1)
    except Exception as e:
        print()
        print(f"Setup failed with error: {e}")
        import traceback
        traceback.print_exc()
        sys.exit(1)


if __name__ == "__main__":
    main()



"""FastAPI application initialization and configuration."""
from __future__ import annotations

import asyncio
import mimetypes
import os
import time
import warnings
from pathlib import Path
from datetime import datetime

from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import HTMLResponse, RedirectResponse
from fastapi.staticfiles import StaticFiles

from searchat.core.watcher import ConversationWatcher
from searchat.core.logging_config import setup_logging, get_logger
from searchat.core.progress import LoggingProgressAdapter
from searchat.api.dependencies import (
    initialize_services,
    start_background_warmup,
    get_config,
    get_indexer,
    get_search_engine,
    get_watcher,
    set_watcher,
    watcher_stats,
    indexing_state,
)
import searchat.api.dependencies as deps
from searchat.api.readiness import get_readiness
from searchat.api.routers import (
    search_router,
    conversations_router,
    bookmarks_router,
    stats_router,
    indexing_router,
    backup_router,
    admin_router,
    status_router,
    chat_router,
    queries_router,
    code_router,
    docs_router,
    dashboards_router,
)
from searchat.config.constants import (
    DEFAULT_HOST,
    DEFAULT_PORT,
    PORT_SCAN_RANGE,
    ENV_PORT,
    ENV_HOST,
    ERROR_INVALID_PORT,
    ERROR_PORT_IN_USE,
)

warnings.filterwarnings(
    "ignore",
    category=UserWarning,
    module=r"multiprocessing\.resource_tracker",
)

_warn_filter = "ignore::UserWarning:multiprocessing.resource_tracker"
_existing_warn = os.environ.get("PYTHONWARNINGS", "")
if _warn_filter not in _existing_warn:
    os.environ["PYTHONWARNINGS"] = (
        f"{_existing_warn},{_warn_filter}" if _existing_warn else _warn_filter
)

# Ensure correct JS MIME types across platforms (notably Windows).
# Without this, browsers may refuse to load ES modules from /static.
mimetypes.add_type("text/javascript", ".js")
mimetypes.add_type("text/javascript", ".mjs")


# Cache HTML at module load for faster responses
_HTML_PATH = Path(__file__).parent.parent / "web" / "index.html"
_CACHED_HTML = _HTML_PATH.read_text(encoding='utf-8')


# Create FastAPI app
app = FastAPI(
    title="Searchat API",
    description="Semantic search for AI coding agent conversations",
    version="0.2.0",
)

# CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Mount static files
static_path = Path(__file__).parent.parent / "web" / "static"
app.mount("/static", StaticFiles(directory=str(static_path)), name="static")

# Mount docs directory for infographics
docs_path = Path(__file__).parent.parent.parent.parent / "docs"
if docs_path.exists():
    app.mount("/docs", StaticFiles(directory=str(docs_path)), name="docs")

# Register routers
app.include_router(search_router, prefix="/api", tags=["search"])
app.include_router(conversations_router, prefix="/api", tags=["conversations"])
app.include_router(bookmarks_router, prefix="/api", tags=["bookmarks"])
app.include_router(stats_router, prefix="/api", tags=["statistics"])
app.include_router(indexing_router, prefix="/api", tags=["indexing"])
app.include_router(backup_router, prefix="/api/backup", tags=["backup"])
app.include_router(admin_router, prefix="/api", tags=["admin"])
app.include_router(status_router, prefix="/api", tags=["status"])
app.include_router(chat_router, prefix="/api", tags=["chat"])
app.include_router(queries_router, prefix="/api", tags=["queries"])
app.include_router(code_router, prefix="/api", tags=["code"])
app.include_router(docs_router, prefix="/api", tags=["docs"])
app.include_router(dashboards_router, prefix="/api", tags=["dashboards"])


def on_new_conversations(file_paths: list[str]) -> None:
    """Callback when watcher detects new conversation files."""
    global projects_cache, watcher_stats, indexing_state

    logger = get_logger(__name__)
    logger.info(f"Auto-indexing {len(file_paths)} new conversations")

    try:
        indexer = get_indexer()
        search_engine = get_search_engine()

        # Mark indexing in progress
        indexing_state["in_progress"] = True
        indexing_state["operation"] = "watcher"
        indexing_state["started_at"] = datetime.now().isoformat()
        indexing_state["files_total"] = len(file_paths)
        indexing_state["files_processed"] = 0

        # Use logging-based progress for background task
        progress = LoggingProgressAdapter()

        enable_adaptive = False
        try:
            enable_adaptive = bool(
                getattr(indexer, "config", None)
                and getattr(indexer.config, "indexing", None)
                and indexer.config.indexing.enable_adaptive_indexing
            )
        except Exception:
            enable_adaptive = False

        if enable_adaptive and hasattr(indexer, "index_adaptive"):
            stats = indexer.index_adaptive(file_paths, progress)
        else:
            stats = indexer.index_append_only(file_paths, progress)

        updated_conversations = getattr(stats, "updated_conversations", 0)
        if stats.new_conversations > 0 or updated_conversations > 0:
            deps.invalidate_search_index()

            watcher_stats["indexed_count"] += stats.new_conversations + updated_conversations
            watcher_stats["last_update"] = datetime.now().isoformat()

            logger.info(
                f"Indexed {stats.new_conversations} new conversations and "
                f"updated {updated_conversations} conversations "
                f"in {stats.update_time_seconds:.2f}s"
            )
    except Exception as e:
        logger.error(f"Failed to index new conversations: {e}")
    finally:
        # Mark indexing complete
        indexing_state["in_progress"] = False
        indexing_state["operation"] = None


@app.on_event("startup")
async def startup_event():
    """Initialize services and start the file watcher on server startup."""
    started = time.perf_counter()
    # IMPORTANT: Initialize services FIRST (loads config)
    initialize_services()

    # Start warmup in background (non-blocking)
    start_background_warmup()

    # THEN get config and setup logging
    config = get_config()
    setup_logging(config.logging)
    logger = get_logger(__name__)

    if os.getenv("SEARCHAT_PROFILE_STARTUP") == "1":
        elapsed_ms = (time.perf_counter() - started) * 1000.0
        logger.info("Startup: initialize_services + schedule warmup %.1fms", elapsed_ms)

    # Start file watcher in background (do not block startup).
    asyncio.create_task(_start_watcher_background(config))

    if os.getenv("SEARCHAT_PROFILE_STARTUP") == "1":
        elapsed_ms = (time.perf_counter() - started) * 1000.0
        logger.info("Startup: total startup_event %.1fms", elapsed_ms)


async def _start_watcher_background(config):
    readiness = get_readiness()
    readiness.set_watcher("starting")

    logger = get_logger(__name__)
    try:
        indexer = get_indexer()
        watcher = ConversationWatcher(
            config=config,
            on_update=on_new_conversations,
            batch_delay_seconds=5.0,
            debounce_seconds=2.0,
        )

        indexed_paths = await asyncio.to_thread(indexer.get_indexed_file_paths)
        watcher.set_indexed_files(indexed_paths)

        watcher.start()
        set_watcher(watcher)

        readiness.set_watcher("running")
        logger.info(
            f"Live watcher started, monitoring {len(watcher.get_watched_directories())} directories"
        )
    except Exception as e:
        readiness.set_watcher("error", error=str(e))
        logger.error(f"Failed to start watcher: {e}")


@app.on_event("shutdown")
async def shutdown_event():
    """Stop the file watcher on server shutdown."""
    watcher = get_watcher()
    if watcher:
        watcher.stop()
        set_watcher(None)


@app.get("/", response_class=HTMLResponse)
async def root():
    """Serve the main HTML page."""
    return HTMLResponse(_CACHED_HTML)


@app.get("/favicon.ico", include_in_schema=False)
async def favicon_ico():
    """Redirect default browser favicon request to our SVG favicon."""
    return RedirectResponse(url="/static/favicon.svg")


@app.get("/conversation/{conversation_id}", response_class=HTMLResponse)
async def serve_conversation_page(conversation_id: str):
    """Serve HTML page for viewing a specific conversation."""
    # For now, serve the same main page (it handles conversation viewing via client-side routing)
    return HTMLResponse(_CACHED_HTML)


def main():
    """Run the server with configurable host and port."""
    import uvicorn
    import socket
    import warnings
    import threading
    import time
    import webbrowser
    import sys

    prog = Path(sys.argv[0]).name
    argv = set(sys.argv[1:])
    if prog.startswith("searchat-web"):
        if "--version" in argv:
            from searchat import __version__

            print(__version__)
            return
        if "-h" in argv or "--help" in argv:
            print("Usage: searchat-web")
            print()
            print("Environment variables:")
            print(f"  {ENV_HOST}=<host>   (default: {DEFAULT_HOST})")
            print(f"  {ENV_PORT}=<port>   (default: auto-scan {PORT_SCAN_RANGE[0]}-{PORT_SCAN_RANGE[1]})")
            print("  SEARCHAT_OPEN_BROWSER=0   Disable opening the browser tab")
            print()
            return

    # Python 3.12 can emit a noisy multiprocessing.resource_tracker warning on
    # shutdown with some native deps (e.g., torch/sentence-transformers).
    warnings.filterwarnings(
        "ignore",
        message=r"resource_tracker: There appear to be .* leaked semaphore objects to clean up at shutdown",
        category=UserWarning,
    )

    # Get host from environment or use default
    host = os.getenv(ENV_HOST, DEFAULT_HOST)

    # Get port from environment or scan for available port
    env_port = os.getenv(ENV_PORT)
    if env_port:
        try:
            port = int(env_port)
            if not (1 <= port <= 65535):
                print(ERROR_INVALID_PORT.format(port=port))
                return
        except ValueError:
            print(ERROR_INVALID_PORT.format(port=env_port))
            return
    else:
        # Scan for available port in range
        port, max_port = PORT_SCAN_RANGE

        while port <= max_port:
            try:
                # Test if port is available
                with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
                    s.bind((host, port))
                # Port is available
                break
            except OSError:
                port += 1

        if port > max_port:
            print(ERROR_PORT_IN_USE.format(
                start=PORT_SCAN_RANGE[0],
                end=PORT_SCAN_RANGE[1],
                port=port
            ))
            return

    print(f"Starting Searchat server...")
    print(f"  URL: http://localhost:{port}")
    print(f"  Host: {host}")
    print(f"  Port: {port}")
    print()
    print("Press Ctrl+C to stop")

    open_browser_raw = os.getenv("SEARCHAT_OPEN_BROWSER", "1").strip().lower()
    open_browser = open_browser_raw not in {"0", "false", "no", "off"}
    if "--no-browser" in argv:
        open_browser = False
    interactive = sys.stdout.isatty()

    def _open_browser_when_ready() -> None:
        url_host = host
        if host in {"0.0.0.0", "::", ""}:
            url_host = "localhost"
        url = f"http://{url_host}:{port}"

        probe_host = "127.0.0.1" if url_host == "localhost" else url_host

        deadline = time.time() + 10.0
        while time.time() < deadline:
            try:
                with socket.create_connection((probe_host, port), timeout=0.25):
                    break
            except OSError:
                time.sleep(0.1)
        else:
            print(f"Warning: server did not start within 10s; not opening browser ({url})")
            return

        try:
            webbrowser.open_new_tab(url)
        except Exception as exc:
            print(f"Warning: failed to open browser tab: {exc}")

    if open_browser and interactive:
        threading.Thread(target=_open_browser_when_ready, daemon=True).start()

    uvicorn.run(app, host=host, port=port)


if __name__ == "__main__":
    main()



"""API tests for conversation export endpoints."""
from __future__ import annotations

import json
import pytest
from datetime import datetime
from unittest.mock import Mock, patch
from fastapi.testclient import TestClient

from searchat.api.app import app
from searchat.api.models.responses import ConversationMessage


@pytest.fixture
def client():
    """FastAPI test client."""
    return TestClient(app)


@pytest.fixture
def mock_conversation_response():
    """Mock conversation response data."""
    mock = Mock()
    mock.conversation_id = "conv-123"
    mock.title = "Test Conversation"
    mock.project_id = "test-project"
    mock.project_path = "/path/to/project"
    mock.tool = "claude"
    mock.message_count = 2
    mock.messages = [
        ConversationMessage(
            role="user",
            content="Hello, how are you?",
            timestamp="2026-01-28T10:00:00"
        ),
        ConversationMessage(
            role="assistant",
            content="I'm doing well, thank you!",
            timestamp="2026-01-28T10:00:05"
        )
    ]
    return mock


def test_export_conversation_json(client, mock_conversation_response):
    """Test GET /api/conversation/{id}/export with JSON format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.get("/api/conversation/conv-123/export?format=json")

        assert response.status_code == 200
        assert response.headers["content-type"] == "application/json"
        assert 'attachment; filename="conv-123.json"' in response.headers["content-disposition"]

        data = json.loads(response.content)
        assert data["conversation_id"] == "conv-123"
        assert data["title"] == "Test Conversation"
        assert data["project_id"] == "test-project"
        assert data["project_path"] == "/path/to/project"
        assert data["tool"] == "claude"
        assert data["message_count"] == 2
        assert len(data["messages"]) == 2
        assert data["messages"][0]["role"] == "user"
        assert data["messages"][0]["content"] == "Hello, how are you?"


def test_export_conversation_markdown(client, mock_conversation_response):
    """Test GET /api/conversation/{id}/export with Markdown format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.get("/api/conversation/conv-123/export?format=markdown")

        assert response.status_code == 200
        assert "text/markdown" in response.headers["content-type"]
        assert 'attachment; filename="conv-123.md"' in response.headers["content-disposition"]

        content = response.content.decode('utf-8')
        assert "# Test Conversation" in content
        assert "**Conversation ID:** conv-123" in content
        assert "**Project:** test-project" in content
        assert "**Project Path:** /path/to/project" in content
        assert "**Tool:** claude" in content
        assert "**Messages:** 2" in content
        assert "## Message 1 - USER" in content
        assert "Hello, how are you?" in content
        assert "## Message 2 - ASSISTANT" in content
        assert "I'm doing well, thank you!" in content


def test_export_conversation_text(client, mock_conversation_response):
    """Test GET /api/conversation/{id}/export with text format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.get("/api/conversation/conv-123/export?format=text")

        assert response.status_code == 200
        assert response.headers["content-type"] == "text/plain; charset=utf-8"
        assert 'attachment; filename="conv-123.txt"' in response.headers["content-disposition"]

        content = response.content.decode('utf-8')
        assert "CONVERSATION: Test Conversation" in content
        assert "ID: conv-123" in content
        assert "Project: test-project" in content
        assert "Project Path: /path/to/project" in content
        assert "Tool: claude" in content
        assert "Messages: 2" in content
        assert "[Message 1 - USER]" in content
        assert "Hello, how are you?" in content
        assert "[Message 2 - ASSISTANT]" in content
        assert "I'm doing well, thank you!" in content


def test_export_conversation_default_format(client, mock_conversation_response):
    """Test export endpoint defaults to JSON format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.get("/api/conversation/conv-123/export")

        assert response.status_code == 200
        assert response.headers["content-type"] == "application/json"


def test_export_conversation_invalid_format(client, mock_conversation_response):
    """Test export endpoint rejects invalid format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.get("/api/conversation/conv-123/export?format=xml")

        assert response.status_code == 400
        assert "Invalid format" in response.json()["detail"]


def test_export_conversation_not_found(client):
    """Test export endpoint handles nonexistent conversation."""
    from fastapi import HTTPException

    def mock_get_conversation(conversation_id):
        raise HTTPException(status_code=404, detail="Conversation not found")

    with patch("searchat.api.routers.conversations.get_conversation", side_effect=mock_get_conversation):
        response = client.get("/api/conversation/nonexistent/export")

        assert response.status_code == 404


def test_export_conversation_without_project_path(client):
    """Test export handles conversations without project path."""
    mock = Mock()
    mock.conversation_id = "conv-456"
    mock.title = "Test"
    mock.project_id = "project"
    mock.project_path = None  # No project path
    mock.tool = "vibe"
    mock.message_count = 1
    mock.messages = [ConversationMessage(role="user", content="Test", timestamp="2026-01-28T10:00:00")]

    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock):
        # Test markdown (has conditional project path)
        response = client.get("/api/conversation/conv-456/export?format=markdown")
        assert response.status_code == 200
        content = response.content.decode('utf-8')
        assert "**Project Path:**" not in content

        # Test text (has conditional project path)
        response = client.get("/api/conversation/conv-456/export?format=text")
        assert response.status_code == 200
        content = response.content.decode('utf-8')
        assert "Project Path:" not in content


def test_export_conversation_case_insensitive_format(client, mock_conversation_response):
    """Test export format parameter is case-insensitive."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        # Test uppercase
        response = client.get("/api/conversation/conv-123/export?format=JSON")
        assert response.status_code == 200
        assert "application/json" in response.headers["content-type"]

        # Test mixed case
        response = client.get("/api/conversation/conv-123/export?format=MarkDown")
        assert response.status_code == 200
        assert "text/markdown" in response.headers["content-type"]


def test_bulk_export_json(client, mock_conversation_response):
    """Test POST /api/conversations/bulk-export with JSON format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.post(
            "/api/conversations/bulk-export",
            json={
                "conversation_ids": ["conv-1", "conv-2"],
                "format": "json"
            }
        )

        assert response.status_code == 200
        assert response.headers["content-type"] == "application/zip"
        assert "attachment; filename=" in response.headers["content-disposition"]
        assert ".zip" in response.headers["content-disposition"]


def test_bulk_export_markdown(client, mock_conversation_response):
    """Test POST /api/conversations/bulk-export with markdown format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.post(
            "/api/conversations/bulk-export",
            json={
                "conversation_ids": ["conv-1"],
                "format": "markdown"
            }
        )

        assert response.status_code == 200
        assert response.headers["content-type"] == "application/zip"


def test_bulk_export_text(client, mock_conversation_response):
    """Test POST /api/conversations/bulk-export with text format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.post(
            "/api/conversations/bulk-export",
            json={
                "conversation_ids": ["conv-1", "conv-2", "conv-3"],
                "format": "text"
            }
        )

        assert response.status_code == 200
        assert response.headers["content-type"] == "application/zip"


def test_bulk_export_empty_list(client):
    """Test bulk export rejects empty conversation list."""
    response = client.post(
        "/api/conversations/bulk-export",
        json={
            "conversation_ids": [],
            "format": "json"
        }
    )

    assert response.status_code == 400
    assert "No conversation IDs" in response.json()["detail"]


def test_bulk_export_too_many_conversations(client):
    """Test bulk export enforces maximum limit."""
    response = client.post(
        "/api/conversations/bulk-export",
        json={
            "conversation_ids": [f"conv-{i}" for i in range(101)],
            "format": "json"
        }
    )

    assert response.status_code == 400
    assert "Too many conversations" in response.json()["detail"]


def test_bulk_export_invalid_format(client):
    """Test bulk export rejects invalid format."""
    response = client.post(
        "/api/conversations/bulk-export",
        json={
            "conversation_ids": ["conv-1"],
            "format": "xml"
        }
    )

    assert response.status_code == 400
    assert "Invalid format" in response.json()["detail"]


def test_bulk_export_default_format(client, mock_conversation_response):
    """Test bulk export defaults to JSON format."""
    with patch("searchat.api.routers.conversations.get_conversation", return_value=mock_conversation_response):
        response = client.post(
            "/api/conversations/bulk-export",
            json={
                "conversation_ids": ["conv-1"]
            }
        )

        assert response.status_code == 200


def test_bulk_export_validation(client):
    """Test bulk export request validation."""
    # Missing conversation_ids
    response = client.post(
        "/api/conversations/bulk-export",
        json={"format": "json"}
    )
    assert response.status_code == 422

    # Invalid JSON
    response = client.post(
        "/api/conversations/bulk-export",
        data="invalid json",
        headers={"Content-Type": "application/json"}
    )
    assert response.status_code == 422



"""DuckDB-on-demand queries over the parquet index.

This module is used by API endpoints that should be fast even when the search
engine/model are still warming.
"""

from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from pathlib import Path


ConversationSort = str


@dataclass(frozen=True)
class IndexStatistics:
    total_conversations: int
    total_messages: int
    avg_messages: float
    total_projects: int
    earliest_date: str | None
    latest_date: str | None


class DuckDBStore:
    def __init__(self, search_dir: Path, *, memory_limit_mb: int | None = None) -> None:
        self._search_dir = search_dir
        self._conversations_dir = search_dir / "data" / "conversations"
        self._memory_limit_mb = memory_limit_mb

    def _conversation_parquets(self) -> list[Path]:
        if not self._conversations_dir.exists():
            return []
        return sorted(self._conversations_dir.glob("*.parquet"))

    def _connect(self):
        import duckdb

        con = duckdb.connect(database=":memory:")
        if self._memory_limit_mb is not None:
            con.execute(f"PRAGMA memory_limit='{int(self._memory_limit_mb)}MB'")
        return con

    def list_projects(self) -> list[str]:
        parquets = self._conversation_parquets()
        if not parquets:
            return []

        con = self._connect()
        try:
            # Project only the required column.
            query = """
            SELECT DISTINCT project_id
            FROM parquet_scan(?)
            WHERE message_count > 0
            ORDER BY project_id
            """
            rows = con.execute(query, [str(self._conversations_dir / "*.parquet")]).fetchall()
            return [r[0] for r in rows]
        finally:
            con.close()

    def list_project_summaries(self) -> list[dict]:
        parquets = self._conversation_parquets()
        if not parquets:
            return []

        con = self._connect()
        try:
            query = """
            SELECT
              project_id,
              COUNT(*)::BIGINT AS conversation_count,
              COALESCE(SUM(message_count), 0)::BIGINT AS message_count,
              MAX(updated_at) AS updated_at
            FROM parquet_scan(?)
            WHERE message_count > 0
            GROUP BY project_id
            ORDER BY project_id
            """
            rows = con.execute(query, [str(self._conversations_dir / "*.parquet")]).fetchall()
            summaries = []
            for project_id, conv_count, msg_count, updated_at in rows:
                updated_at_str = updated_at.isoformat() if isinstance(updated_at, datetime) else str(updated_at)
                summaries.append(
                    {
                        "project_id": project_id,
                        "conversation_count": int(conv_count),
                        "message_count": int(msg_count),
                        "updated_at": updated_at_str,
                    }
                )
            return summaries
        finally:
            con.close()

    def list_conversations(
        self,
        *,
        sort_by: ConversationSort = "length",
        project_id: str | None = None,
        date_from: datetime | None = None,
        date_to: datetime | None = None,
        tool: str | None = None,
        limit: int | None = None,
        offset: int = 0,
    ) -> list[dict]:
        parquets = self._conversation_parquets()
        if not parquets:
            return []

        order_by = {
            "length": "message_count DESC",
            "date_newest": "updated_at DESC",
            "date_oldest": "updated_at ASC",
            "title": "title ASC",
        }.get(sort_by, "message_count DESC")

        conditions = ["message_count > 0"]
        params: list[object] = [str(self._conversations_dir / "*.parquet")]

        if project_id:
            conditions.append("project_id = ?")
            params.append(project_id)

        if tool:
            tool_value = tool.lower()
            if tool_value == "opencode":
                conditions.append("project_id LIKE 'opencode-%'")
            elif tool_value == "vibe":
                conditions.append("project_id LIKE 'vibe-%'")
            elif tool_value == "claude":
                conditions.append("project_id NOT LIKE 'opencode-%'")
                conditions.append("project_id NOT LIKE 'vibe-%'")

        if date_from:
            conditions.append("updated_at >= ?")
            params.append(date_from)

        if date_to:
            conditions.append("updated_at < ?")
            params.append(date_to)

        where_clause = " AND ".join(conditions)

        con = self._connect()
        try:
            query = f"""
            SELECT
              conversation_id,
              project_id,
              title,
              created_at,
              updated_at,
              message_count,
              file_path,
              full_text
            FROM parquet_scan(?)
            WHERE {where_clause}
            ORDER BY {order_by}
            """

            if limit is not None:
                query += "\nLIMIT ?\nOFFSET ?"
                params.append(int(limit))
                params.append(int(offset))
            rows = con.execute(query, params).fetchall()
            columns = [
                "conversation_id",
                "project_id",
                "title",
                "created_at",
                "updated_at",
                "message_count",
                "file_path",
                "full_text",
            ]
            return [dict(zip(columns, row)) for row in rows]
        finally:
            con.close()

    def count_conversations(
        self,
        *,
        project_id: str | None = None,
        date_from: datetime | None = None,
        date_to: datetime | None = None,
        tool: str | None = None,
    ) -> int:
        parquets = self._conversation_parquets()
        if not parquets:
            return 0

        conditions = ["message_count > 0"]
        params: list[object] = [str(self._conversations_dir / "*.parquet")]

        if project_id:
            conditions.append("project_id = ?")
            params.append(project_id)

        if tool:
            tool_value = tool.lower()
            if tool_value == "opencode":
                conditions.append("project_id LIKE 'opencode-%'")
            elif tool_value == "vibe":
                conditions.append("project_id LIKE 'vibe-%'")
            elif tool_value == "claude":
                conditions.append("project_id NOT LIKE 'opencode-%'")
                conditions.append("project_id NOT LIKE 'vibe-%'")

        if date_from:
            conditions.append("updated_at >= ?")
            params.append(date_from)

        if date_to:
            conditions.append("updated_at < ?")
            params.append(date_to)

        where_clause = " AND ".join(conditions)

        con = self._connect()
        try:
            query = f"""
            SELECT COUNT(*)::BIGINT
            FROM parquet_scan(?)
            WHERE {where_clause}
            """
            row = con.execute(query, params).fetchone()
            if row is None:
                return 0
            return int(row[0])
        finally:
            con.close()

    def validate_parquet_scan(self) -> None:
        """Validate DuckDB can connect and scan parquet files.

        If no conversation parquets exist yet, this validates the DuckDB runtime
        only (connection + simple query).
        """
        con = self._connect()
        try:
            con.execute("SELECT 1").fetchone()

            parquets = self._conversation_parquets()
            if parquets:
                con.execute(
                    "SELECT 1 FROM parquet_scan(?) LIMIT 1",
                    [str(self._conversations_dir / "*.parquet")],
                ).fetchone()
        finally:
            con.close()

    def get_conversation_meta(self, conversation_id: str) -> dict | None:
        parquets = self._conversation_parquets()
        if not parquets:
            return None

        con = self._connect()
        try:
            query = """
            SELECT
              conversation_id,
              project_id,
              title,
              created_at,
              updated_at,
              message_count,
              file_path
            FROM parquet_scan(?)
            WHERE conversation_id = ?
            LIMIT 1
            """
            row = con.execute(query, [str(self._conversations_dir / "*.parquet"), conversation_id]).fetchone()
            if row is None:
                return None
            columns = [
                "conversation_id",
                "project_id",
                "title",
                "created_at",
                "updated_at",
                "message_count",
                "file_path",
            ]
            return dict(zip(columns, row))
        finally:
            con.close()


    def get_conversation_record(self, conversation_id: str) -> dict | None:
        """Return full conversation record including messages from parquet."""
        parquets = self._conversation_parquets()
        if not parquets:
            return None

        con = self._connect()
        try:
            query = """
            SELECT
              conversation_id,
              project_id,
              title,
              created_at,
              updated_at,
              message_count,
              file_path,
              messages
            FROM parquet_scan(?)
            WHERE conversation_id = ?
            LIMIT 1
            """
            row = con.execute(query, [str(self._conversations_dir / "*.parquet"), conversation_id]).fetchone()
            if row is None:
                return None
            columns = [
                "conversation_id",
                "project_id",
                "title",
                "created_at",
                "updated_at",
                "message_count",
                "file_path",
                "messages",
            ]
            return dict(zip(columns, row))
        finally:
            con.close()

    def get_statistics(self) -> IndexStatistics:
        parquets = self._conversation_parquets()
        if not parquets:
            return IndexStatistics(
                total_conversations=0,
                total_messages=0,
                avg_messages=0.0,
                total_projects=0,
                earliest_date=None,
                latest_date=None,
            )

        con = self._connect()
        try:
            query = """
            SELECT
              COUNT(*)::BIGINT AS total_conversations,
              COALESCE(SUM(message_count), 0)::BIGINT AS total_messages,
              COALESCE(AVG(message_count), 0)::DOUBLE AS avg_messages,
              COUNT(DISTINCT project_id)::BIGINT AS total_projects,
              MIN(created_at) AS earliest_date,
              MAX(updated_at) AS latest_date
            FROM parquet_scan(?)
            """
            row = con.execute(query, [str(self._conversations_dir / "*.parquet")]).fetchone()
            if row is None:
                return IndexStatistics(
                    total_conversations=0,
                    total_messages=0,
                    avg_messages=0.0,
                    total_projects=0,
                    earliest_date=None,
                    latest_date=None,
                )

            earliest = row[4]
            latest = row[5]

            def _to_iso(value):
                if value is None:
                    return None
                if isinstance(value, datetime):
                    return value.isoformat()
                return str(value)

            return IndexStatistics(
                total_conversations=int(row[0]),
                total_messages=int(row[1]),
                avg_messages=float(row[2]),
                total_projects=int(row[3]),
                earliest_date=_to_iso(earliest),
                latest_date=_to_iso(latest),
            )
        finally:
            con.close()
